--[[----------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------
--
-- TOOL NAME: GT16-Colors
-- VERSIÓN: 1.2b8 (for R3.1)
-- COMPATIBILITY: Windows, Mac, Linux (32/64bit)
-- AUTHOR: ulneiz (Spain)
-- DATE: January 2017
--
--------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------]]--------------------------

--------------------------------------------------------------------------------
-- INITIALS & CONSTANTS
--------------------------------------------------------------------------------

dialog = nil -- Placeholder for the dialog
vb = nil
vb = renoise.ViewBuilder() -- ViewBuilder for show_dialog() function & others


T, F = true, false

--------------------------------------------------------------------------------
--INITIAL, PREFERENCES ...& LOG PRINT
--------------------------------------------------------------------------------
--Initial for manifest.xml
class 'RenoiseScriptingTool' ( renoise.Document.DocumentNode )

function RenoiseScriptingTool:__init()    
  renoise.Document.DocumentNode.__init( self ) 
  self:add_property( 'Name' , 'Untitled Tool' )
  self:add_property( 'Id' , 'Unknown Id' )
  self:add_property( 'Version', 'Unknown Version')
end

manifest = RenoiseScriptingTool()
ok,err = manifest:load_from( 'manifest.xml' )
tool_name = manifest:property( 'Name' ).value
tool_id = manifest:property( 'Id' ).value
tool_version = manifest:property('Version').value

--Preferences for preferences.xml
class 'gt16_preferences'( renoise.Document.DocumentNode )

function gt16_preferences:__init()
  renoise.Document.DocumentNode.__init( self )
  self:add_property( "autostart",    renoise.Document.ObservableBoolean( F ) )
  self:add_property( "startup_prefs",renoise.Document.ObservableBoolean( F ) )
  ---
  self:add_property( "color_blend",  renoise.Document.ObservableBoolean( F ) )
  self:add_property( "number_blend", renoise.Document.ObservableNumber( 15 ) )
  self:add_property( "pattern",      renoise.Document.ObservableNumber( 1 ) )
  self:add_property( "sequence",     renoise.Document.ObservableNumber( 1 ) )
  self:add_property( "metronome",    renoise.Document.ObservableNumber( 1 ) )
  self:add_property( "midi",         renoise.Document.ObservableNumber( 1 ) )
  ---
  self:add_property( "bpm",          renoise.Document.ObservableBoolean( F ) )
  self:add_property( "number_bpm",   renoise.Document.ObservableNumber( 176 ) )
  self:add_property( "lpb",          renoise.Document.ObservableBoolean( F ) )
  self:add_property( "number_lpb",   renoise.Document.ObservableNumber( 8 ) )
  self:add_property( "no_lines",     renoise.Document.ObservableBoolean( F ) )
  self:add_property( "note_col",     renoise.Document.ObservableNumber( 1 ) )
  self:add_property( "number_lines", renoise.Document.ObservableNumber( 96 ) )
  self:add_property( "play",         renoise.Document.ObservableBoolean( F ) )
  ---
  self:add_property( "preferences",  renoise.Document.ObservableBoolean( F ) )
  self:add_property( "bar",          renoise.Document.ObservableBoolean( F ) )
  self:add_property( "up_selector",  renoise.Document.ObservableNumber( 1 ) ) -- 1-4
  self:add_property( "dw_selector",  renoise.Document.ObservableNumber( 1 ) ) -- 1-2
  self:add_property( "at_selector",  renoise.Document.ObservableNumber( 1 ) ) -- 1-10
end

local prefs = gt16_preferences()
renoise.tool().preferences = prefs

--Log print
function LOG(...) print (...) end


--------------------------------------------------------------------------------
-- REQUIRE FILES
--------------------------------------------------------------------------------
require ("keyhandler")
require ("tools/tool_01")
require ("tools/tool_02")
require ("tools/tool_03")
require ("tools/tool_04")
require ("tools/tool_05")
require ("tools/tool_06")
require ("tools/tool_07")
require ("tools/tool_08")
require ("tools/tool_09")
require ("tools/tool_10")


--------------------------------------------------------------------------------
-- NAVIGATION SWITCHES
--------------------------------------------------------------------------------
--Up Selector Switch
function up_select( i )
  LOG( 'up_select: ', i )
  vb.views['up_select_cls'].visible = ( i == 1 )
  vb.views['up_select_hlp'].visible = ( i == 2 )
  vb.views['up_select_abt'].visible = ( i == 3 )
  vb.views['up_select_tls'].visible = ( i == 4 )
end 

--Down Selector Switch
function dw_select( i )
  LOG( 'dw_select: ', i )
  vb.views['dw_select_t'].visible = ( i == 1 )
  vb.views['dw_select_f'].visible = ( i == 2 )
end

--Advanced Tools Selector Switch
function at_select( i )
  LOG( 'at_select: ', i )
  vb.views['at_select_01'].visible = ( i == 1 )
  vb.views['at_select_02'].visible = ( i == 2 )
  vb.views['at_select_03'].visible = ( i == 3 )
  vb.views['at_select_04'].visible = ( i == 4 )
  vb.views['at_select_05'].visible = ( i == 5 )
  vb.views['at_select_06'].visible = ( i == 6 )
  vb.views['at_select_07'].visible = ( i == 7 )
  vb.views['at_select_08'].visible = ( i == 8 )
  vb.views['at_select_09'].visible = ( i == 9 )
  vb.views['at_select_10'].visible = ( i ==10 )
end


--------------------------------------------------------------------------------
-- PREFERENCES
--------------------------------------------------------------------------------
--Preferences
function show_preferences( i )
  LOG( 'show_cbox_pref()' , i )
  vb.views.cbox_pref_t.visible = ( i == T )
  vb.views.cbox_pref_f.visible = ( i == F )
end
checkbox_pref = vb:row { --Show/Hide Preferences
  vb:text { text = 'Preferences ' },
  vb:checkbox { id = 'cbox_pref', bind = prefs.preferences, notifier = function(i) show_preferences(i) end, tooltip ="Show/hide preferences.\nWork also at the start." }
}

--Bar
function show_cbox_bar( i )
  LOG( 'show_cbox_bar()' , i )
  vb.views.cbox_bar_t.visible = ( i == T )
  vb.views.cbox_bar_f.visible = ( i == F )
end
checkbox_bar = vb:checkbox { id = 'cbox_bar', value = F, notifier = function(i) show_cbox_bar(i) end, tooltip = "Show/hide the 'Navigate Tracks' bar bellow.\nWork with 'AutoBar' preference." }
  --Activate bar ( prevent a error starting Renoise )
function max_tracks( n )
  if vb.views['cbox_bar'].value == T then
    n = #renoise.song().tracks
    return n
  else
    return 8
  end
end
function value_tracks( sti )
  if vb.views['cbox_bar'].value == T then
    sti = renoise.song().selected_track_index
    return sti
  else
    return 1
  end
end
--Autobar
checkbox_autobar = vb:row {
  vb:text { text = 'AutoBar' },
  vb:checkbox { id = 'cbox_autobar', value = prefs.bar.value, bind = prefs.bar, tooltip = "Show/hide the 'Navigate Tracks' bar at the start.\nWork with 'Autostart' preference." }
}
function autobar()
  if vb.views['cbox_autobar'].value == T then
    vb.views['cbox_bar'].value = T
  else
    vb.views['cbox_bar'].value = F
  end
end

--Autoplay
checkbox_autoplay = vb:column { style = 'panel', margin = 4, width = 58,
  vb:text { text = 'AutoPlay' },
  vb:vertical_aligner { mode = 'distribute', height = 20,
    vb:checkbox { id = 'cbox_autoplay', value = prefs.play.value, bind = prefs.play, tooltip = "AutoPlay song at the start" }
  }
}

function autoplay( song, play_mode )
  if not vb.views['cbox_autoplay'].value == T then
  else
    song = renoise.song()
    play_mode = renoise.Transport.PLAYMODE_RESTART_PATTERN
    song.transport:start( play_mode )
  end
end

--Autostart
autostart = vb:row { --Activate/Deactivate Autostart
  vb:text{ font = 'bold', text = "  Autostart" },
  vb:checkbox { bind = prefs.autostart, tooltip ="Autostart GUI of GT16-Colors.\n[Default = no start]" } --Autostart
}

function first_pattern( song ) --return to sequence_index = 1
  song = renoise.song()
  song.selected_sequence_index = 1
end

--Startup Preferences
startup_prefs = vb:row { --Activate/Deactivate Startup Preferences
  vb:checkbox { bind = prefs.startup_prefs, tooltip ="Startup Preferences (all lower options).\n[Default = no start].\nWork with 'Autostart' preference." }, --Startup Preferences
  vb:text{ font = 'bold', text = "Startup Preferences" }
}

--Color Blend and Value
function auto_color_blend( song, i, n ) --Auto Color Blend (Background Blend)
  if prefs.color_blend.value == F then
    else
      song = renoise.song()
      n = #song.tracks 
      for i = 1, n, 1 do
      --if song.tracks[i].color_blend == 0 then
        song.tracks[i].color_blend = prefs.number_blend.value
      --end
    end
  end
end
color_blend = vb:column { style = 'panel', margin = 4, --Activate/Deactivate Color Blend to Start
  vb:row {
    vb:text{ text = 'Color Blend', width = 62 },
    vb:button { width = 10, height = 18, color = { 0x00,0x40,0x70 }, notifier = function() auto_color_blend() end, tooltip = 'Apply Now!' }
  },
  vb:row {
    vb:vertical_aligner { mode = 'distribute',
      vb:checkbox { value = prefs.color_blend.value, bind = prefs.color_blend, tooltip ="Activate Background Blend to all tracks at the start.\n[Default = disabled]" },
    },
    vb:valuebox { id = 'vbox_blend', width = 54, height = 20, min = 0, max = 99, value = prefs.number_blend.value, bind = prefs.number_blend, tooltip ="Value of general Background Blend at the start.\n[Range = 00-99. Default value = 15]" }
  }
}


--Rename Note Column
function rename_note_col_01( song, column_names )
  song = renoise.song()
  column_names = { 'Note 01' , 'Note 02' , 'Note 03' , 'Note 04' , 'Note 05' , 'Note 06' , 'Note 07' , 'Note 08' , 'Note 09' , 'Note 10' , 'Note 11' , 'Note 12' }
  if vb.views['cb_note_col'].value == F then
    for i = 1, song.sequencer_track_count do --#song.tracks
      --print ('song.sequencer_track_count: ', song.sequencer_track_count)
      --print ('#song.tracks: ', #song.tracks)
      if song.tracks[ i ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
          --local column_names = { 'C  do' , 'C#' , 'D  re' , 'D#' , 'E  mi' , 'F  fa' , 'F#' , 'G  sol' , 'G#' , 'A  la' , 'A#' , 'B  si' }
        for note_column = 1, 12 do
          song.tracks[ i ]:set_column_name( note_column , column_names[ note_column ] )
        end
      end
    end
    else
    if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
      for note_column = 1, 12 do
        song.selected_track:set_column_name( note_column , column_names[ note_column ] )
      end
    end
  end
end
function rename_note_col_02( song, column_names )
  song = renoise.song()
  column_names = { 'Do' , 'Do#' , 'Re' , 'Re#' , 'Mi' , 'Fa' , 'Fa#' , 'Sol' , 'Sol#' , 'La' , 'La#' , 'Si' }
  if vb.views['cb_note_col'].value == F then
    for i = 1, song.sequencer_track_count do
      if song.tracks[ i ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
        for note_column = 1, 12 do
          song.tracks[ i ]:set_column_name( note_column , column_names[ note_column ] )
        end
      end
    end
    else
    if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
      for note_column = 1, 12 do
        song.selected_track:set_column_name( note_column , column_names[ note_column ] )
      end
    end
  end
end
function rename_note_col_03( song, column_names )
  song = renoise.song()
  column_names = { 'C' , 'C#' , 'D' , 'D#' , 'E' , 'F' , 'F#' , 'G' , 'G#' , 'A' , 'A#' , 'B' }
  if vb.views['cb_note_col'].value == F then
    for i = 1, song.sequencer_track_count do
      if song.tracks[ i ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
        for note_column = 1, 12 do
          song.tracks[ i ]:set_column_name( note_column , column_names[ note_column ] )
        end
      end
    end
    else
    if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
      for note_column = 1, 12 do
        song.selected_track:set_column_name( note_column , column_names[ note_column ] )
      end
    end
  end
end
function rename_note_col_04( song, column_names )
  song = renoise.song()
  column_names = { 'C  [Do]' , 'C#' , 'D  [Re]' , 'D#' , 'E  [Mi]' , 'F  [Fa]' , 'F#' , 'G  [Sol]' , 'G#' , 'A  [La]' , 'A#' , 'B  [Si]' }
  if vb.views['cb_note_col'].value == F then
    for i = 1, song.sequencer_track_count do
      if song.tracks[ i ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
        for note_column = 1, 12 do
          song.tracks[ i ]:set_column_name( note_column , column_names[ note_column ] )
        end
      end
    end
    else
    if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
      for note_column = 1, 12 do
        song.selected_track:set_column_name( note_column , column_names[ note_column ] )
      end
    end
  end
end
function rename_note_col_05( song, column_names )
  song = renoise.song()
  column_names = { 'C  [Do] 01', 'C#  02', 'D  [Re] 03 ', 'D#  04', 'E  [Mi] 05', 'F  [Fa] 06', 'F#  07', 'G  [Sol] 08', 'G#  09', 'A  [La] 10', 'A#  11', 'B  [Si] 12',  }
  if vb.views['cb_note_col'].value == F then
    for i = 1, song.sequencer_track_count do
      if song.tracks[ i ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
        for note_column = 1, 12 do
          song.tracks[ i ]:set_column_name( note_column , column_names[ note_column ] )
        end
      end
    end
    else
    if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
      for note_column = 1, 12 do
        song.selected_track:set_column_name( note_column , column_names[ note_column ] )
      end
    end
  end
end
function rename_note_col_06( song, column_names )
  song = renoise.song()
  column_names = { '01 C  [Do]', '02 C#', '03 D  [Re]', '04 D#', '05 E  [Mi]', '06 F  [Fa]', '07 F#', '08 G  [Sol]', '09 G#', '10 A  [La]', '11 A#', '12 B  [Si]',  }
  if vb.views['cb_note_col'].value == F then
    for i = 1, song.sequencer_track_count do
      if song.tracks[ i ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
        for note_column = 1, 12 do
          song.tracks[ i ]:set_column_name( note_column , column_names[ note_column ] )
        end
      end
    end
    else
    if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
      for note_column = 1, 12 do
        song.selected_track:set_column_name( note_column , column_names[ note_column ] )
      end
    end
  end
end
function rename_note_col_07( song, column_names )
  song = renoise.song()
  column_names = { '01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12',  }
  if vb.views['cb_note_col'].value == F then
    for i = 1, song.sequencer_track_count do
      if song.tracks[ i ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
        for note_column = 1, 12 do
          song.tracks[ i ]:set_column_name( note_column , column_names[ note_column ] )
        end
      end
    end
    else
    if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
      for note_column = 1, 12 do
        song.selected_track:set_column_name( note_column , column_names[ note_column ] )
      end
    end
  end
end
function rename_note_col_08( song, column_names ) --'Note'
  song = renoise.song()
  column_names = { 'Note' , 'Note' , 'Note' , 'Note' , 'Note' , 'Note' , 'Note' , 'Note' , 'Note' , 'Note' , 'Note' , 'Note' }
  if vb.views['cb_note_col'].value == F then
    for i = 1, song.sequencer_track_count do
      if song.tracks[ i ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
        for note_column = 1, 12 do
          song.tracks[ i ]:set_column_name( note_column , column_names[ note_column ] )
        end
      end
    end
    else
    if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
      for note_column = 1, 12 do
        song.selected_track:set_column_name( note_column , column_names[ note_column ] )
      end
    end
  end
end

function select_note_col ( i )
  i = vb.views['pp_note_col'].value
  if     i == 1 then return 
  elseif i == 2 then return rename_note_col_01()
  elseif i == 3 then return rename_note_col_02()
  elseif i == 4 then return rename_note_col_03()
  elseif i == 5 then return rename_note_col_04()
  elseif i == 6 then return rename_note_col_05()
  elseif i == 7 then return rename_note_col_06()
  elseif i == 8 then return rename_note_col_07()
  elseif i == 9 then return rename_note_col_08()
  end
end

function bt_note_col_color()
  if vb.views['pp_note_col'].value == 1 then
    vb.views['bt_note_col'].color = { 0x40,0x00,0x00 }
    vb.views['bt_note_col'].active = F
    else
    vb.views['bt_note_col'].color = { 0x00,0x40,0x70 }
    vb.views['bt_note_col'].active = T
  end
end
renoise.tool().app_new_document_observable:add_notifier( bt_note_col_color )

note_col = vb:column { style = 'panel', margin = 4, --Rename Note Column all or individual tracks
  vb:row {
    vb:text{ text = 'Rename Note Columns', width = 142 },
    vb:button { id = 'bt_note_col', width = 10, height = 18, color = { 0x00,0x40,0x70 }, notifier = function() select_note_col() end, tooltip = 'Apply Now!' }
  },
  vb:row {
    vb:popup { id = 'pp_note_col', width = 134, height = 20, value = prefs.note_col.value, bind = prefs.note_col, notifier = function() bt_note_col_color() end,
      tooltip = 'Rename Note Column to all tracks or selected track only.\n[Default = No Mod.]',
      items = {
        " No Mod."," Note 01 until Note 12"," Do until Si"," C until B"," C [Do] until B [Si]"," C [Do] 01 until B [Si] 12"," 01 C [Do] until 12 B [Si]"," 01 until 12"," Note"
      }
    },
    vb:vertical_aligner { mode = 'distribute',
      vb:checkbox { id = 'cb_note_col', value = false, tooltip = 'Rename Note Column for selected track or all tracks.\nEnabled = selected track.\nDisabled = all tracks.' }
    }
  }
}


--Pattern Templates
function auto_pattern( song, sti, stc )
  song = renoise.song()
  sti = song.selected_track_index
  stc = song.sequencer_track_count
  song.selected_track_index = stc --replacement position
  if vb.views['pp_pattern'].value == 2 then pest_fn_A03_01() end -- Symphony Orchestra Big
  if vb.views['pp_pattern'].value == 3 then pest_fn_A03_02() end -- Symphony Orchestra Little
  if vb.views['pp_pattern'].value == 4 then pest_fn_A03_03() end -- Symphony Orchestra Little + Triple Piano
  if vb.views['pp_pattern'].value == 5 then pest_fn_A03_04() end -- Symphony Orchestra Little + Synths + Pianos
  song.selected_track_index = 1
  for sti = stc, 1, -1 do --erase 8 to 1 tracks (to inverse)
    song:delete_track_at( sti )
  end
end

function select_pattern( i )
  i = vb.views['pp_pattern'].value
  if     i == 1 then return 
  elseif i >  1 then auto_pattern()
  end
end

function bt_pattern_color()
  if vb.views['pp_pattern'].value == 1 then
    vb.views['bt_pattern'].color = { 0x40,0x00,0x00 }
    vb.views['bt_pattern'].active = F
    else
    vb.views['bt_pattern'].color = { 0x00,0x40,0x70 }
    vb.views['bt_pattern'].active = T
  end
end
renoise.tool().app_new_document_observable:add_notifier( bt_pattern_color )

template = vb:column { style = 'panel', margin = 4, --Templates
  vb:row {
    vb:text{ text = 'Pattern Template (Tracks)', width = 130 },
    vb:button { id = 'bt_pattern', width = 10, height = 18, color = { 0x00,0x40,0x70 }, notifier = function() select_pattern() end, tooltip = 'Apply Now!' }
  },
  vb:popup { id = 'pp_pattern', width = 140, height = 20, value = prefs.pattern.value, bind = prefs.pattern, notifier = function() bt_pattern_color() end, tooltip = 'Select a Pattern Template at the start.\n[Default = Nothing]',
    items = {
      'Nothing','S. Orchestra Big','S. Orchestra Little','SO + Triple Piano','SO + Synths + Pianos'
    }
  }
}

--Sequence Template
function select_sequence ( i )
  i = vb.views['pp_sequence'].value
  if     i == 1 then return 
  elseif i == 2 then for i = 1, 4  do amic_CH02_3A() end
  elseif i == 3 then for i = 1, 8  do amic_CH02_3A() end
  elseif i == 4 then for i = 1, 12 do amic_CH02_3A() end
  elseif i == 5 then for i = 1, 16 do amic_CH02_3A() end
  elseif i == 6 then for i = 1, 20 do amic_CH02_3A() end
  elseif i == 7 then for i = 1, 24 do amic_CH02_3A() end
  elseif i == 8 then for i = 1, 28 do amic_CH02_3A() end
  elseif i == 9 then for i = 1, 32 do amic_CH02_3A() end
  elseif i ==10 then for i = 1, 36 do amic_CH02_3A() end
  elseif i ==11 then for i = 1, 40 do amic_CH02_3A() end
  elseif i ==12 then for i = 1, 44 do amic_CH02_3A() end
  elseif i ==13 then for i = 1, 48 do amic_CH02_3A() end
  elseif i ==14 then for i = 1, 52 do amic_CH02_3A() end
  elseif i ==15 then for i = 1, 56 do amic_CH02_3A() end
  elseif i ==16 then for i = 1, 60 do amic_CH02_3A() end
  elseif i ==17 then for i = 1, 64 do amic_CH02_3A() end
  end
end

function bt_sequence_color()
  if vb.views['pp_sequence'].value == 1 then
    vb.views['bt_sequence'].color = { 0x40,0x00,0x00 }
    vb.views['bt_sequence'].active = F
    else
    vb.views['bt_sequence'].color = { 0x00,0x40,0x70 }
    vb.views['bt_sequence'].active = T
  end
end
renoise.tool().app_new_document_observable:add_notifier( bt_sequence_color )

sequence = vb:column { style = 'panel', margin = 4,
  vb:row {
    vb:text{ text = 'Sequence Template', width = 130 },
    vb:button { id = "bt_sequence", width = 10, height = 18, color = { 0x00,0x40,0x70 }, notifier = function() select_sequence() end, tooltip = 'Apply Now!' }
  },
  vb:popup { id = "pp_sequence", width = 140, height = 20, value = prefs.sequence.value, bind = prefs.sequence, notifier = function() bt_sequence_color() end, tooltip = 'Select a Sequence Template at the start.\n[Default = Nothing]',
    items = {
      'Nothing','[Section x4] x4',  '[Section x4] x8',  '[Section x4] x12', '[Section x4] x16', '[Section x4] x20',
                '[Section x4] x24', '[Section x4] x28', '[Section x4] x32', '[Section x4] x36', '[Section x4] x40',
                '[Section x4] x44', '[Section x4] x48', '[Section x4] x52', '[Section x4] x56', '[Section x4] x60',
                '[Section x4] x64',
    }
  }
}

--Metronome
function select_metronome ( i )
  i = vb.views['pp_metronome'].value
  if     i == 1 then return 
  elseif i == 2 then return amic_CH03_4A()
  end
end

function bt_metronome_color()
  if vb.views['pp_metronome'].value == 1 then
    vb.views['bt_metronome'].color = { 0x40,0x00,0x00 }
    vb.views['bt_metronome'].active = F
    else
    vb.views['bt_metronome'].color = { 0x00,0x40,0x70 }
    vb.views['bt_metronome'].active = T
  end
end
renoise.tool().app_new_document_observable:add_notifier( bt_metronome_color )

metronome = vb:column { style = 'panel', margin = 4,
  vb:row {
    vb:text{ text = 'Metronome (Tr Phrase)', width = 130 },
    vb:button { id = 'bt_metronome', width = 10, height = 18, color = { 0x00,0x40,0x70 }, notifier = function() select_metronome() end, tooltip = 'Apply Now!' }
  },
  vb:popup { id = 'pp_metronome', width = 140, height = 20, value = prefs.metronome.value, bind = prefs.metronome, notifier = function() bt_metronome_color() end, tooltip = 'Select a Metronome at the start.\n[Default = Nothing]',
    items = {
      'Nothing','Metronome [3x4]'
    }
  }
}

--MIDI Mapping File
function select_midi ( i )
  i = vb.views['pp_midi'].value
  if     i == 1 then return 
  elseif i == 2 then return renoise.song():load_midi_mappings( "xrnm/midi-input.xrnm" )
  end
end

function bt_midi_color()
  if vb.views['pp_midi'].value == 1 then
    vb.views['bt_midi'].color = { 0x40,0x00,0x00 }
    vb.views['bt_midi'].active = F
    else
    vb.views['bt_midi'].color = { 0x00,0x40,0x70 }
    vb.views['bt_midi'].active = T
  end
end
renoise.tool().app_new_document_observable:add_notifier( bt_midi_color )

midi = vb:column { style = 'panel', margin = 4,
  vb:row {
    vb:text{ text = 'MIDI Mapping Profile', width = 140 },
    vb:button { id ='bt_midi', width = 10, height = 18, color = { 0x00,0x40,0x70 }, notifier = function() select_midi() end, tooltip = 'Apply Now!' }
  },
  vb:popup { id = 'pp_midi', width = 150, height = 20, value = prefs.midi.value, bind = prefs.midi, notifier = function() bt_midi_color() end, tooltip = 'Select a Midi Mapping Profile at the start.\n[Default = Nothing]',
    items = {
      'Nothing','Novation Launchkey 61'
    }
  }
}

--BPM (Beats Per Minute)
function auto_bpm( song )
  song = renoise.song()
  if prefs.bpm.value == F then
    else
    song.transport.bpm = prefs.number_bpm.value --176 --BPM (32-999)
  end
end    
bpm = vb:column { style = 'panel', margin = 4, --Activate/Deactivate BPM to Start
  vb:row {
    vb:text{ text = 'BPM', width = 62 },
    vb:button { width = 10, height = 18, color = { 0x00,0x40,0x70 }, notifier = function() auto_bpm() end, tooltip = 'Apply Now!' }
  },
  vb:row {
    vb:vertical_aligner { mode = 'distribute',
    vb:checkbox { value = prefs.bpm.value, bind = prefs.bpm, tooltip = "Activate Beats Per Minute (BPM) at the start.\n[Default = disabled]" },
    },
    vb:valuebox { id = 'vbox_bpm', width = 54, height = 20, min = 32, max = 999, value = prefs.number_bpm.value, bind = prefs.number_bpm,
      tooltip ="Value of Beats Per Minute (BPM) at the start.\n[Range = 32-999. Default value = 176]"
    }
  }
}

--LPB (Lines Per Beat)
function auto_lpb( song )
  song = renoise.song()
  if prefs.lpb.value == F then
    else
    song.transport.lpb = prefs.number_lpb.value --8 --LPB (1-256)
  end
end    
lpb = vb:column { style = 'panel', margin = 4, --Activate/Deactivate LPB to Start
  vb:row {
    vb:text{ text = 'LPB', width = 62 },
    vb:button { width = 10, height = 18, color = { 0x00,0x40,0x70 }, notifier = function() auto_lpb() end, tooltip = 'Apply Now!' }
  },
  vb:row {
    vb:vertical_aligner { mode = 'distribute',
      vb:checkbox { value = prefs.lpb.value, bind = prefs.lpb, tooltip = "Activate Lines Per Beat (LPB) at the start.\n[Default = disabled]" },
    },
    vb:valuebox { id = 'vbox_lpb', width = 54, height = 20, min = 1, max = 256, value = prefs.number_lpb.value, bind = prefs.number_lpb, tooltip ="Value of Lines Per Beat (LPB) at the start.\n[Range = 1-256. Default value = 8]" }
  }
}

--Number of Lines
function auto_no_lines( song, i )
  song = renoise.song()
  if prefs.no_lines.value == F then
    else
    for i = 1, #song.patterns do--#song.sequencer.pattern_sequence, 1 do
      --local pattern = song:pattern( i )     
      song.patterns[i].number_of_lines = prefs.number_lines.value --96
    end
  end
end    
no_lines = vb:column { style = 'panel', margin = 4, --Activate/Deactivate Color Blend to Start
  vb:row {
    vb:text{ text = 'Lines', width = 62 },
    vb:button { width = 10, height = 18, color = { 0x00,0x40,0x70 }, notifier = function() auto_no_lines() end, tooltip = 'Apply Now!' }
  },
  vb:row {
    vb:vertical_aligner { mode = 'distribute',
      vb:checkbox { value = prefs.no_lines.value, bind = prefs.no_lines,
        tooltip = "Activate Number of Lines to all patterns at the start.\n[Default = disabled]"
      }
    },
    vb:valuebox { id = 'vbox_lines', width = 54, height = 20, min = 1, max = 512, value = prefs.number_lines.value, bind = prefs.number_lines,
      tooltip ="Value of Number of Lines to all patterns at the start.\n[Range = 1-512. Default value = 96]"
    }
  }
}


--Button Apply Preferences Now!
function apply_preferences()
  auto_color_blend( prefs.color_blend )
  select_pattern( prefs.pattern )
  select_sequence( prefs.sequence )
  auto_bpm( prefs.bpm )
  auto_lpb( prefs.lpb )
  auto_no_lines( prefs.no_lines )
  select_metronome( prefs.metronome )
  select_midi( prefs.midi )
  select_note_col( prefs.note_col )
  first_pattern()
  autoplay()
end

bt_apply_preferences = vb:horizontal_aligner { width = 248, mode = 'right',
  vb:button { text = 'Apply All\nPreferences Now!', width = 110, height = 47, color = { 0x00,0x40,0x70 },
    notifier = function() apply_preferences() end, tooltip = 'Apply All Preferences Now!\nOnly suitable to start a new song.\nUse with caution!!!'
  }
}

--------------------------------------------------------------------------------
-- SWITCHES & BUTTONS
--------------------------------------------------------------------------------
SWI_UP_SELECT = vb:switch {
  id = 'SWI_UP_SELECT', items = { '⇱', '☲K', '?', '⚒' }, tooltip = '⇱ Close Down  [CTRL + END]\n☲K Key Commands  [CTRL + PRIOR]\n? About  [CTRL + NEXT]\n⚒ Advanced Tools [CTRL + INS]',
  width = 102, height = 24, value = prefs.up_selector.value, bind = prefs.up_selector, notifier = function(i) up_select(i) end
}

SWI_DW_SELECT = vb:switch {
  id = 'SWI_DW_SELECT', items = { '☱', '⇲' }, tooltip = "☱ Controls Above  [CTRL + ']\n⇲ Close Up  [CTRL + ¡]",
  width = 59, height = 20, value = prefs.dw_selector.value, bind = prefs.dw_selector, notifier = function(i) dw_select(i) end
}

SWI_AT_SELECT = vb:switch {
  id = 'SWI_AT_SELECT', items = { '1', '2', '3', '4', '5', '6', '7', '8', '9', '0' }, tooltip = 'Advanced Tools Selector\n[CTRL + 1,2...9,0]\n1.\n2.\n3.\n4.\n5.\n6.\n7.\n8.\n9.\n0.',
  width = 300, height = 20, value = prefs.at_selector.value, bind = prefs.at_selector, notifier = function(i) at_select(i) end
}

-- Lower Buttons ( x14 ) ------------------------
-------------------------------------------------

--[[
--Return Parent Index
function parent_index( song, sti, parent_index )
  song = renoise.song()
  sti = song.selected_track_index
  parent_index = nil
  for i, track in ripairs (song.tracks ) do
    if track.type == renoise.Track.TRACK_TYPE_GROUP then
      if i - #track.members <= sti and sti < i then
        parent_index = i
      end
    end
  end
  print( 'parent_index:', parent_index )
  return parent_index
end
]]


--Swap, Undo/Redo, NavTracks, Del & First/Last
function navt_swap_left( song, type, sti, m, n, parent_index )
  song = renoise.song()
  type = song.selected_track.type
  sti = song.selected_track_index

  --For track
  if type == renoise.Track.TRACK_TYPE_SEQUENCER then
    --left out of the group
    if song.tracks[ sti ].group_parent then
      m = #song.tracks[ sti ].group_parent.members
      print('m: ', m)

      --Parent Index
      for i, track in ripairs ( song.tracks ) do
        if track.type == renoise.Track.TRACK_TYPE_GROUP then
          if i - #track.members <= sti and sti < i then
            parent_index = i
          end
        end
      end
      --For first member and rest
      if sti == parent_index - m then
        song:insert_track_at( sti )
        song:swap_tracks_at( sti + 1, sti )
        song:delete_track_at( sti + 1 )
        song:select_previous_track()
        --if previeus is track
        else
        if sti > 1 then
          if song.tracks[ sti - 1 ].type ~= renoise.Track.TRACK_TYPE_GROUP then
            song:swap_tracks_at( sti, sti - 1 )
            song:select_previous_track()
          end
        --if previeus is group
          if song.tracks[ sti - 1 ].type == renoise.Track.TRACK_TYPE_GROUP then
            song:add_track_to_group( sti, sti - 1 )
            song:select_previous_track()
          end
        end
      end

      else
      --left track swap track out of group
      if sti > 1 then
        if song.tracks[ sti - 1 ].type == renoise.Track.TRACK_TYPE_GROUP then
          song:add_track_to_group( sti, sti - 1 )
          song:select_previous_track()
          else
          song:swap_tracks_at( sti, sti - 1 )
          song:select_previous_track()
        end
      end
    end
  end
  --- --- ---
  --For Group
  if type == renoise.Track.TRACK_TYPE_GROUP then
    n = #song.tracks[ sti ].members
    print('n: ', n)
    if song.tracks[ sti ].group_parent then
      m = #song.tracks[ sti ].group_parent.members
      print('m: ', m)

      --Parent Index
      for i, track in ripairs ( song.tracks ) do
        if track.type == renoise.Track.TRACK_TYPE_GROUP then
          if i - #track.members <= sti and sti < i then
            parent_index = i
          end
        end
      end
      --For first member and rest
      if sti == parent_index - m + n then
        song:insert_track_at( sti - n )
        song:swap_tracks_at( sti + 1, sti - n )
        song:delete_track_at( sti + 1 )
        song:select_previous_track()
        --if previeus is track
        elseif song.tracks[ sti - n - 1 ].type ~= renoise.Track.TRACK_TYPE_GROUP then
        song:swap_tracks_at( sti, sti - n - 1 )
        song:select_previous_track()
        --if previeus is group
        elseif song.tracks[ sti - n - 1 ].type == renoise.Track.TRACK_TYPE_GROUP then
        song:add_track_to_group( sti, sti -n - 1 )
        song:select_previous_track()
      end
      
      else
      --left track swap track out of group
      if sti > n + 1 then
        if song.tracks[ sti - n - 1 ].type == renoise.Track.TRACK_TYPE_GROUP then --
          song:add_track_to_group( sti, sti - n - 1 )
          song:select_previous_track()
          else
          song:swap_tracks_at( sti, sti - n - 1 )
          song:select_previous_track()
        end
      end
    end
  end
end

---

function navt_swap_right( song, type, sti, sti2, m, n, parent_index )
  song = renoise.song()
  type = song.selected_track.type
  sti = song.selected_track_index
  --For Track
  if type == renoise.Track.TRACK_TYPE_SEQUENCER and sti < song.sequencer_track_count then
    if song.tracks[ sti + 1 ].group_parent then
      m = #song.tracks[ sti + 1 ].group_parent.members

      --Parent Index
      for i, track in ripairs ( song.tracks ) do
        if track.type == renoise.Track.TRACK_TYPE_GROUP then
          if i - #track.members <= sti + 1 and sti + 1 < i then
            parent_index = i
          end
        end
      end

      print( 'sti: ', sti )
      print( 'parent_index: ', parent_index )
      print( 'm: ', m )

      if sti > parent_index - m - 1 and song.tracks[ sti + 1 ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
        song:swap_tracks_at( sti + 1, sti )
        song:select_next_track()
      end
      if sti == parent_index - m - 1 then
        if song.tracks[ sti + 1 ].type == renoise.Track.TRACK_TYPE_GROUP and song.tracks[ sti + 2 ].type == renoise.Track.TRACK_TYPE_GROUP then
          song:add_track_to_group( sti, sti + 1 )
          elseif song.tracks[ sti + 1 ].type == renoise.Track.TRACK_TYPE_GROUP and song.tracks[ sti + 2 ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
            song:add_track_to_group( sti, parent_index - m )
            song.selected_track_index = sti
          else
          song:add_track_to_group( sti, parent_index - m )
          song:delete_track_at( sti + 2 )
          song.selected_track_index = sti
        end
        elseif song.tracks[ sti + 1 ].type == renoise.Track.TRACK_TYPE_GROUP then
        song:insert_track_at( sti + 2 )
        song:swap_tracks_at( sti, sti + 2 )
        song:delete_track_at( sti )
        song.selected_track_index = sti + 1
      end
      else
      if song.tracks[ sti + 1 ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
        song:swap_tracks_at( sti, sti + 1 )
        song:select_next_track()
        else
        song:insert_track_at( sti + 2 )
        song:swap_tracks_at( sti, sti + 2 )
        song:delete_track_at( sti )
        song.selected_track_index = sti + 1
      end
    end
  end

  --For Group
  if type == renoise.Track.TRACK_TYPE_GROUP and sti < song.sequencer_track_count then
    n = #song.tracks[ sti ].members
    print('n: ', n)
    if song.tracks[ sti + 1 ].group_parent then
      m = #song.tracks[ sti + 1 ].group_parent.members

      --Parent Index
      for i, track in ripairs ( song.tracks ) do
        if track.type == renoise.Track.TRACK_TYPE_GROUP then
          if i - #track.members <= sti + 1 and sti + 1 < i then
            parent_index = i
          end
        end
      end

      print( 'sti: ', sti )
      print( 'parent_index: ', parent_index )
      print( 'm: ', m )

      if sti > parent_index - m - 1 and song.tracks[ sti + 1 ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
        song:swap_tracks_at( sti + 1, sti )
        song:select_next_track()
      elseif song.tracks[ sti + 1 ].type == renoise.Track.TRACK_TYPE_GROUP then
        song:insert_track_at( sti + 2 )
        song:swap_tracks_at( sti, sti + 2 )
        song:delete_track_at( sti - n )
        song.selected_track_index = sti + 1      
      end
      --- ---
      if sti == parent_index - m - 1 then
        if song.tracks[ sti + 1 ].type == renoise.Track.TRACK_TYPE_GROUP and song.tracks[ sti + 2 ].type == renoise.Track.TRACK_TYPE_GROUP then
            --song:add_track_to_group( sti, sti + 1 )
          elseif song.tracks[ sti + 1 ].type == renoise.Track.TRACK_TYPE_GROUP and song.tracks[ sti + 2 ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
            song:add_track_to_group( sti, parent_index - m )
            song.selected_track_index = sti
          else
          song:add_track_to_group( sti, parent_index - m ) --ok
          song:delete_track_at( sti + 2 )
          song.selected_track_index = sti
        end
      end
      --- ---
      else
      if song.tracks[ sti + 1 ].type == renoise.Track.TRACK_TYPE_SEQUENCER then
        song:swap_tracks_at( sti, sti + 1 )
        song:select_next_track()
        else
        song:insert_track_at( sti + 2 )
        song:swap_tracks_at( sti, sti + 2 )
        song:delete_track_at( sti - n )
        song.selected_track_index = sti + 1
      end

    end
  end
end

---
function navt_coll_gr( song, A )
  song = renoise.song()
  if song.transport.single_track_edit_mode == T then
    song.transport.single_track_edit_mode = F
  end
  local selected_track = song.tracks[ song.selected_track_index ]
  if ( selected_track.type == renoise.Track.TRACK_TYPE_GROUP ) then
    for i = 1, #selected_track.members do
      A = selected_track.members[ i ]
          if ( A.collapsed == T ) then A.collapsed = F -- False is expanded
        renoise.app():show_status( "GT16-Colors:  selected Group expanded." )
      elseif ( A.collapsed == F ) then A.collapsed = T -- True is collapsed
          renoise.app():show_status( "GT16-Colors:  selected Group collapsed." )
      end
    end
  end
  if ( selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER ) or
     ( selected_track.type == renoise.Track.TRACK_TYPE_MASTER ) or
   ( selected_track.type == renoise.Track.TRACK_TYPE_SEND ) then
    if selected_track.collapsed == T then song.tracks[song.selected_track_index].collapsed = F
    if ( selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER ) then
      renoise.app():show_status( "GT16-Colors:  selected Track expanded." )
      end
    if ( selected_track.type == renoise.Track.TRACK_TYPE_MASTER ) then
      renoise.app():show_status( "GT16-Colors:  selected Master expanded." )
      end
    if ( selected_track.type == renoise.Track.TRACK_TYPE_SEND ) then
      renoise.app():show_status( "GT16-Colors:  selected Send expanded." )
    end
    elseif selected_track.collapsed == F then song.tracks[song.selected_track_index].collapsed = T
    if ( selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER ) then
      renoise.app():show_status( "GT16-Colors:  selected Track colapsed." )
    end
    if ( selected_track.type == renoise.Track.TRACK_TYPE_MASTER ) then
      renoise.app():show_status( "GT16-Colors:  selected Master colapsed." )
    end
    if ( selected_track.type == renoise.Track.TRACK_TYPE_SEND ) then
      renoise.app():show_status( "GT16-Colors:  selected Send colapsed." )
    end
    end
  end  
end
---
function navt_on_off( song, A )
  song = renoise.song()
  if ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER ) or 
     ( song.selected_track.type == renoise.Track.TRACK_TYPE_GROUP ) or
     ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEND ) then
    A = song.tracks[song.selected_track_index].mute_state
  end
  if A == 1 then song.tracks[song.selected_track_index]:mute()
    if (song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER) then
      renoise.app():show_status( "GT16-Colors:  selected Track Off." )
  end
  if (song.selected_track.type == renoise.Track.TRACK_TYPE_GROUP) then
      renoise.app():show_status( "GT16-Colors:  selected Group Off." )
  end
  if (song.selected_track.type == renoise.Track.TRACK_TYPE_SEND) then
      renoise.app():show_status( "GT16-Colors:  selected Send Off." )
  end
  elseif A == 2 then song.tracks[song.selected_track_index]:unmute()
    if (song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER) then
      renoise.app():show_status( "GT16-Colors:  selected Track Play." )
  end
  if (song.selected_track.type == renoise.Track.TRACK_TYPE_GROUP) then
      renoise.app():show_status( "GT16-Colors:  selected Group Play." )
  end
  if (song.selected_track.type == renoise.Track.TRACK_TYPE_SEND) then
      renoise.app():show_status( "GT16-Colors:  selected Send Play." )
  end
  else 
  renoise.app():show_status( "GT16-Colors:  please, for Play/Off select before a Track/Group/Send in Pattern Editor or Mixer!!!" )
  end
end
---
function navt_undo()
  local song = renoise.song()
  song:undo()
end
function navt_redo()
  local song = renoise.song()
  song:redo()
end
---

function navt_rem( song, sti, stc )
  song = renoise.song()
  sti = song.selected_track_index
  stc = song.sequencer_track_count
  if ( stc <= 250 ) then ---- LIMIT = 250 ---------|***
    for sti = stc, 2, -1 do
      song:delete_track_at( sti )
    end
    song.selected_track_index = 1
  renoise.app():show_status( "GT16-Colors:  all Groups & Tracks removed inside Pattern Editor, least the first. Start again!" )
  elseif stc == sti then
    renoise.app():show_status( "GT16-Colors:  only a Track or Group in Pattern Editor!" )
  end
end
---
function navt_del( song )
  song = renoise.song()
  if ( song.sequencer_track_count == 1 ) then
  elseif ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER ) or
         ( song.selected_track.type == renoise.Track.TRACK_TYPE_GROUP ) then
    song:delete_track_at( song.selected_track_index )
    renoise.app():show_status( "GT16-Colors:  selected Track/Group deleted." )
  end     
  if ( song.sequencer_track_count + song.send_track_count  == 1 ) then
  elseif ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEND ) then
    song:delete_track_at( song.selected_track_index )
    renoise.app():show_status( "GT16-Colors:  selected send deleted." )
  end
  if ( song.selected_track.type == renoise.Track.TRACK_TYPE_MASTER ) then
    renoise.app():show_status( "GT16-Colors:  Master is not deletable! Please, for delete select before a Track/Group/Send in Pattern Editor or Mixer!!!" )
  end 
  if ( song.selected_track_index == 1 ) and ( song.sequencer_track_count == 1 ) then
  renoise.app():show_status( "GT16-Colors:  only a Track or Group in Pattern Editor!" )      
  end
end
---
function navt_clr( song, spi, sti ) --clear selected track (& automation)
  song = renoise.song()
  spi, sti = song.selected_pattern_index, song.selected_track_index
  song.patterns[ spi ].tracks[ sti ]:clear()
end

---
function repeat_navt_02(release)
  if not release then
    if renoise.tool():has_timer(repeat_navt_02) then
      navt_02()
      renoise.tool():remove_timer(repeat_navt_02)
      renoise.tool():add_timer(navt_02, 40)
    else
      navt_02()
      renoise.tool():add_timer(repeat_navt_02, 400)
    end
  else
    if renoise.tool():has_timer(repeat_navt_02) then
      renoise.tool():remove_timer(repeat_navt_02)
    elseif renoise.tool():has_timer(navt_02) then
      renoise.tool():remove_timer(navt_02)
    end
  end  
end
function repeat_navt_04(release)
  if not release then
    if renoise.tool():has_timer(repeat_navt_04) then
      navt_04()
      renoise.tool():remove_timer(repeat_navt_04)
      renoise.tool():add_timer(navt_04, 40)
    else
      navt_04()
      renoise.tool():add_timer(repeat_navt_04, 400)
    end
  else
    if renoise.tool():has_timer(repeat_navt_04) then
      renoise.tool():remove_timer(repeat_navt_04)
    elseif renoise.tool():has_timer(navt_04) then
      renoise.tool():remove_timer(navt_04)
    end
  end  
end
function navt_01( song )
  song = renoise.song()
  if song.selected_track_index > 2 then
    song:select_previous_track() song:select_previous_track()
  renoise.app():show_status( "GT16-Colors: x2 previous Track/Group selected." )
  end  
end
function navt_02( song )
  song = renoise.song()
  if song.selected_track_index > 1 then
    song:select_previous_track()
  renoise.app():show_status( "GT16-Colors:  previous Track/Group selected." )
  end
end
function navt_03( song )
  song = renoise.song()
  if song.selected_track_index < song.sequencer_track_count - 1 then
    song:select_next_track() song:select_next_track()
  renoise.app():show_status( "GT16-Colors:  x2 next Track/Group selected." )
  end
end
function navt_04( song )
  song = renoise.song()
  if song.selected_track_index < song.sequencer_track_count then
    song:select_next_track()
  renoise.app():show_status( "GT16-Colors:  next Track/Group selected." )
  end
end
---

function navt_first( song )
  song = renoise.song()
  song.selected_track_index = 1
  renoise.app():show_status( "GT16-Colors:  jumped to first Track/Group." )
end
function navt_last( song )
  song = renoise.song()
  song.selected_track_index = song.sequencer_track_count
  renoise.app():show_status( "GT16-Colors:  jumped to last Track/Group." )
end
---
function repeat_navs_up( release )
  if not release then
    if renoise.tool():has_timer( repeat_navs_up ) then
      navs_up()
      renoise.tool():remove_timer( repeat_navs_up )
      renoise.tool():add_timer( navs_up, 40 )
    else
      navs_up()
      renoise.tool():add_timer( repeat_navs_up, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_navs_up ) then
      renoise.tool():remove_timer( repeat_navs_up )
    elseif renoise.tool():has_timer( navs_up ) then
      renoise.tool():remove_timer( navs_up )
    end
  end  
end
function repeat_navs_down( release )
  if not release then
    if renoise.tool():has_timer( repeat_navs_down ) then
      navs_down()
      renoise.tool():remove_timer( repeat_navs_down )
      renoise.tool():add_timer( navs_down, 40 )
    else
      navs_down()
      renoise.tool():add_timer( repeat_navs_down, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_navs_down ) then
      renoise.tool():remove_timer( repeat_navs_down )
    elseif renoise.tool():has_timer( navs_down ) then
      renoise.tool():remove_timer( navs_down )
    end
  end  
end
function navs_up( song, ssi )
  song = renoise.song()
  ssi = song.selected_sequence_index
  if song.selected_sequence_index > 1 then
    song.selected_sequence_index = ssi - 1 --select next pattern
    renoise.app():show_status( "GT16-Colors:  previous Pattern." )
  end
end
function navs_down( song, ssi )
  song = renoise.song()
  ssi = song.selected_sequence_index
  if #song.sequencer.pattern_sequence > ( ssi ) then
    song.selected_sequence_index = ssi + 1 --select next pattern
    renoise.app():show_status( "GT16-Colors:  next Pattern." )
  end
end
function navs_top( song )
  song = renoise.song()
  song.selected_sequence_index = 1
end
function navs_bottom( song, ssi )
  song = renoise.song()
  song.selected_sequence_index = #song.sequencer.pattern_sequence
end
---
function play_song( song, play_mode )
  song = renoise.song()
  play_mode = renoise.Transport.PLAYMODE_RESTART_PATTERN
  song.transport:start(play_mode)
  renoise.app():show_status( "GT16-Colors:  Song play." )
end

function stop_song( song )
  song = renoise.song()
  song.transport:stop()
  renoise.app():show_status( "GT16-Colors:  Song stop." )
end


-- Show Counters in real time --
-------------------------------------------------
function counters( song )
  song = renoise.song()
  local function ini_counters()
    vb.views["pt_index"].text = ( "Idx [%s" ):format( song.selected_sequence_index ) --selected pattern
    vb.views["tr_index"].text = ( "x %s]" ):format( song.selected_track_index ) --selected track
    vb.views["index_value"].text = ( "%s" ):format( song.selected_track_index ) --selected track
    
    --- ---
    vb.views["pt_count"].text = ( " Tt [%s" ):format( #song.sequencer.pattern_sequence ) --total patterns
    vb.views["tr_count"].text = ( "x %s]" ):format( song.sequencer_track_count ) --total tracks
    --- ---
    vb.views["tr_name"].text = ( "%s" ):format( song.tracks[ song.selected_track_index ].name ) --Name selected track
    vb.views["tr_color"].color = ( song.tracks[ song.selected_track_index ].color ) --Name selected track
    --color
    --- ---
    if song.sequencer_track_count <= 48 then
      vb.views["bar_count"].color = { 0x00,0xFF,0x2A } --green
      elseif song.sequencer_track_count >= 49 and song.sequencer_track_count <= 96 then
      vb.views["bar_count"].color = { 0xFF,0x80,0x00 } --orange
      elseif song.sequencer_track_count >= 97 then
      vb.views["bar_count"].color = { 0xFF,0x00,0x00 } --red
    end
    --- ---
    vb.views["tr_send"].text = ( "Snd %s" ):format( song.send_track_count )--Total Sends
    --- ---
    vb.views["nav_track"].max = #song.tracks
    vb.views["nav_track"].value = song.selected_track_index --bar selected track
  end
  song.selected_sequence_index_observable:add_notifier(ini_counters) 
  song.selected_track_index_observable:add_notifier(ini_counters)

  song.sequencer.pattern_sequence_observable:add_notifier(ini_counters)
  song.tracks[song.selected_track_index].name_observable:add_notifier(ini_counters)
  song.tracks[song.selected_track_index].color_observable:add_notifier(ini_counters)
end

renoise.tool().app_new_document_observable:add_notifier(counters) -- To initialice New Start Renoise

function wmp_rename_track( song ) --rename track
  song = renoise.song()
  song.tracks[song.selected_track_index].name = vb.views["tr_name"].text
end

  
-- Text (Key Commands & About) -----------------
------------------------------------------------

txt_K00 = vb:text {
text =
"* ALT+CTRL + G = Invoke GT16-Colors. Assign manually this Key Command in 'Preferences/Keys... Global/Tools/GT16-Colors'"
}

txt_K01 = vb:text {
text =
"F1 = 'Main Groups' Switch\n"..
"F2 = 'Groups' Switch\n"..
"F3 = 'SubGroups' Switch\n"..
"F4 = 'Tracks' Switch\n"..
"SHIFT + F3 = 'Octaves' Switch\n"..
"\n"..
"← → = Left / Right Select Color\n"..
"↑ ↓ = Up / Down Select Distribution\n"..
"\n"..
"CTRL + PRIOR =  ☲K  Keys info\n"..
"CTRL + NEXT =  ?  About GT16-Colors\n"..
"\n"..
"CTRL + INS =  ⚒  Advanced Tools\n"..
"CTRL + 1,2...9,0 =  Tools Switch\n"..
"CTRL+SHIFT + 2 = SPNO Expand"
}

txt_K02 = vb:text {
text =
"F5 = 'Cmb ...Main Groups' Switch\n"..
"F6 = 'Cmb ...Groups' Switch\n"..
"F7 = 'Cmb ...SubGroups' Switch\n"..
"F8 = 'Cmb ...Tracks' Switch\n"..
"\n"..
"CTRL + Z =  ↩  Undo\n"..
"CTRL + Y =  ↪  Redo\n"..
"\n"..
"CTRL+SHIFT + K =  |≡◄  Coll./Exp. Tr/Gr\n"..
"CTRL+SHIFT + J =  |≡◄  Coll./Exp. Tr in Gr\n"..
"Ç =  ☊  Mute/Unmute Selected Tr/Gr\n"..
"\n"..
"CTRL + HOME =  ⇱  Compact Mode\n"..
"CTRL + END =  ⇱  ... ☲K  ?  ⚒  Closed\n"..
"ESC =  ✖  Exit Window"
}

txt_K03 = vb:text {
text =
"F9   =  |◄  First Tr/Gr Switch\n"..
"F10 =  ◄  Previous Tr/Gr Switch\n"..
"F11 =  ►  Next Tr/Gr Switch\n"..
"F12 =  ►|  Last Tr/Gr Switch\n"..
"\n"..
"CTRL + F9   =  ↶|  Swap Selected Tr/Gr to Left\n"..
"CTRL + F10 =  ◄◄  Previous Tr/Gr x2 Switch\n"..
"CTRL + F11 =  ►►  Next Tr/Gr x2 Switch\n"..
"CTRL + F12 =  |↷  Swap Selected Tr/Gr to Right\n"..
"\n"..
"CTRL+SHIFT + DEL = Remove all Tr/Gr\n"..
"CTRL + DEL = Delete Tr/Gr\n"..
"\n"..
"ALT + SPACE = Insert Selection\n"..
"ALT + RETURN = Process SPNO"
}


txt_A01 = vb:multiline_text {
width = 684, height = 130, text = 
"'GT16-Colors' is a free MultiTool created by ulneiz for Renoise, to build & customice various distributions/combinations of 'Groups & Tracks per Colors' for Pattern Editor & Mixer.\n"..
"There are '16 Colors' with until 1200 Options to choose and multiple Tool Modules for control. Select a 'Color', select a 'Distribution' or 'Combination' and build according your needs. With GT16-Colors, the groups are the protagonists!\n"..
"\n"..
"Use only a floating window to control your building. Use the tabs to enlarge or reduce the window. Use the integrated key commands, your mouse or both for maximum control. Use the aditional controls below of each Advanced Tool to discover a new ocean... \n"..
"\n"..
"⚠ Some 'Combined Options' includes many Groups & Tracks (>100), to insert only with a click of mouse or key command.\n"..
"Please, use with moderation!!! Think before using...\n"..
"\n"..
"⚒ Moreover, GT16-Colors include until ten bottom slots to control & edit (Advanced Tools section). This version 1.2 include two powerful tools, 'GTC' (magnificent for control) & 'SPNO' (effective for sort pitches of notes per octaves) integrated in the slots one & two. Apart, if you know the LUA code language, you can add your personal tools here (eight free slots).\n"..
"\n"..
"This version 1.2 of GT16-Colors is tested with Renoise 3.1.0 and work fine.\n"..
"\n"..
"Enjoy your Renoise!   Enjoy your GT16-Colors!\n"..
"\n"..
"                     ✉ To contact ulneiz: visit the Renoise Forums, www.renoise.com"
}

txt_A02 = vb:text { text = "℗ 2016. ulneiz  (Spain)" }



--------------------------------------------------------------------------------
-- GT16_CONTENT (GUI)
--------------------------------------------------------------------------------
content =
vb:column {
  margin = 4,
  --tabs Controls------------
  vb:column {
    id = 'dw_select_f',
    visible = F,
  },
  ------------------------------------------------------------------------------ GT16-Colors
  vb:column {
    id = 'dw_select_t',
    visible = F,
    --menu-------------------------------------
    vb:column {
      style = 'group',  --body
      width = 704,
      margin = 4,
      --values------------------------------------
      vb:horizontal_aligner {
        mode = 'left',
        vb:horizontal_aligner {
          vb:horizontal_aligner { --Pattern Sequence
            width = 110,
            margin = 2,
            vb:column { 
              tooltip = 'Number of Pattern & Track Selected.\n[Y x X], Y = Pattern Selected (Sequence), X = Track Selected (Index).\nSelect any Track to reactivate!',
              width = 110,
              style = 'body',
              vb:horizontal_aligner {
                height = 21,
                mode = 'center',
                vb: text {
                  id = 'pt_index',
                  align = 'right',
                  font = 'bold',
                  text = 'Index'
                },
                vb: text {
                  id = 'tr_index',
                  align = 'left',
                  font = 'bold',                  
                  text = '[Y x X]'
                }
              }
            }
          },
          vb:horizontal_aligner { width = 6 },
          vb: textfield {
            id = 'tr_name',
            width = 220,
            height = 24,
            align = 'center',
            text = 'Name of Track.',
            notifier = function() wmp_rename_track() end,
            tooltip = 'Rename Track/Group/Master/Send.',
          },
          vb:button { --Counter Tracks Marker
            active = F,
            id = 'tr_color',  -- ID
            color = { 0x20,0x00,0x00 }, -- black 
            tooltip = 'Color Track',
            height = 25,
            width = 24,
          },
          vb:horizontal_aligner { width = 4 },
          vb:horizontal_aligner { --Counter Tracks/Groups
            width = 110,
            margin = 2,
            vb:column {
              tooltip = 'Number of Total Patterns & Tracks in Song [Y x X].\nSelect any Track to reactivate!',
              width = 110,
              style = 'body',
              vb:horizontal_aligner {
                height = 21,
                mode = 'center',
                vb: text {
                  id = 'pt_count',
                  align = 'right',
                  font = 'bold',
                  text = 'Total'
                },
                vb: text {
                  id = 'tr_count',
                  align = 'left',
                  font = 'bold',
                  text = '[Y x X]'
                }
              }
            }
          },
          vb:button { --Counter Tracks Marker
            active = F,
            id = 'bar_count',  -- ID
            color = { 0x00,0x30,0x00 }, -- black 
            tooltip = 'Green: Low Level of Tracks (until 48).\nOrange: Middle Level of Tracks (49 until 96).\nRed: High Level of Tracks (more of 96).',
            height = 25,
            width = 7
          },
          vb:horizontal_aligner { width = 4 },
          vb:horizontal_aligner { --Counter Send
            tooltip = 'Total Number of Send Tracks.\nSelect any Track to reactivate!',
            width = 60,
            margin = 2,
            vb:column {
              width = 60,
              style = 'body',
              vb:horizontal_aligner {
                height = 21,
                mode = 'center',
                vb: text {
                  id = 'tr_send',
                  align = 'center',
                  font = 'bold',
                  text = 'Sends'
                }
              }
            }
          },
          vb:horizontal_aligner { width = 12 },
          checkbox_pref, --Preferences
          vb:horizontal_aligner { width = 12 },
          vb:text { text = 'Bar ' },
          checkbox_bar --Bars
        }
      },    
      --NavTracks-------------------------------
      vb:horizontal_aligner {
        SWI_UP_SELECT, --hmenu
        vb:horizontal_aligner { width = 4 },
        vb:button {
          id = 'play',
          bitmap = 'icons/play-ico.bmp',
          height = 24,
          width = 30,
          color = { 0xC0,0x50,0x00 }, --F68211
          notifier = function()play_song() end,
          tooltip = 'Play song from initial line\n[]'
        },
        vb:button {
          id = 'stop',
          bitmap = 'icons/stop-ico.bmp',
          height = 24,
          width = 30,
          color = { 0xC0,0x50,0x00 },
          notifier = function()stop_song() end,
          tooltip = 'Stop song\n[]'
        },
        vb:horizontal_aligner { width = 4 },
        vb:button {
          id = 'navt_undo',
          bitmap = 'icons/undo-ico.bmp',
          height = 24,
          width = 40,
          color = { 0x8A,0x5A,0x00 },
          notifier = function()navt_undo() end,
          tooltip = 'Undo  \n[CTRL + Z]'
        },
        vb:button {
          id = 'navt_redo',
          bitmap = 'icons/redo-ico.bmp',
          height = 24,
          width = 40,
          color = { 0x8A,0x5A,0x00 },
          notifier = function()navt_redo() end,
          tooltip = 'Redo  \n[CTRL + Y]'
        },
        vb:horizontal_aligner { width = 4 },
        vb:button {
          id = 'navt_cln',
          text = 'REM',
          height = 24,
          width = 40,
          color = { 0x46,0x00,0x00 },
          notifier = function()navt_rem() end,
          tooltip = 'Remove all tracks/groups inside pattern editor (least the first) & start again!\n[CTRL+SHIFT + DEL]\n⚠ Limited until 250 tracks/groups!'
        },
        vb:button {
          id = 'navt_del',
          text = 'DEL',
          height = 24,
          width = 40,
          notifier = function()navt_del() end,
          color = { 0x46,0x00,0x00 },
          tooltip = 'Delete selected track/group/send\n[CTRL + DEL]'
        },
        vb:button {
          id = 'navt_clr',
          text = 'CLR',
          height = 24,
          width = 40,
          color = { 0x66,0x00,0x00 },
          notifier = function()navt_clr() end,
          tooltip = 'Clear selected track/group/master/send & automation\n[ ]'
        },
        vb:horizontal_aligner { width = 4 },
        vb:button {
          id = 'note_on_off',
          bitmap = 'icons/speacker-ico.bmp',
          height = 24,
          width = 30,
          color = { 0x06,0x3B,0x70 },
          notifier = function() navt_on_off() end,
          tooltip = "'Play/Off' selected track/group/send (master not included)\n[Ç]"
        },
        vb:button {
          id = 'collapse_expand_gr',
          bitmap = 'icons/collapse-ico.bmp',
          height = 24,
          width = 30,
          color = { 0x7F,0x05,0x54 },
          notifier = function() navt_coll_gr() end,
          tooltip = "'Collapse/expand' all tracks inside the group parent\n[CTRL+SHIFT + J]\nor\n'Collapse/expand' selected track/master/send\n[CTRL+SHIFT + K]"
        },
        vb:horizontal_aligner { width = 4 },
        vb:button {
          id = 'navt_swap_left',
          bitmap = 'icons/swap-left-ico.bmp',
          height = 24,
          width = 28,
          color = { 0x00,0x20,0x50 },
          notifier = function()navt_swap_left() end,
          tooltip = 'swap selected track/group to left\n[CTRL + F9]'
        },
        vb:button {
          id = 'navt_swap_right',
          bitmap = 'icons/swap-right-ico.bmp',
          height = 24,
          width = 28,
          color = { 0x00,0x20,0x50 },
          notifier = function()navt_swap_right() end,
          tooltip = 'Swap selected track/group to right\n[CTRL + F12]'
        },
        vb:horizontal_aligner { width = 4 },
        vb:button {
          id = 'navt_first',
          bitmap = 'icons/first-ico.bmp',
          height = 24,
          width = 23,
          color = { 0x40,0x00,0x80 },
          notifier = function()navt_first() end,
          tooltip = 'First track/group \n[F9]'
        },
        vb:button {
          id = 'navt_02',
          bitmap = 'icons/left2-ico.bmp',
          height = 24,
          width = 30,
          color = { 0x40,0x00,0x80 },
          --notifier = function()navt_02() end,
          pressed =  function()repeat_navt_02() end,
          released = function()repeat_navt_02(T) end,
          tooltip = 'Previous track/group\n[F10]\nor\nPrevious track/group x2\n[CTRL + F10]'
        },
        vb:button {
          id = 'navt_03',
          bitmap = 'icons/right2-ico.bmp',
          height = 24,
          width = 30,
          color = { 0x40,0x00,0x80 },
          --notifier = function()navt_04() end,
          pressed =  function()repeat_navt_04() end,
          released = function()repeat_navt_04(T) end,
          tooltip = 'Next track/group\n[F11]\nor\nNext track/group x2\n[CTRL + F11]'
        },
        vb:button {
          id = 'navt_last',
          bitmap = 'icons/last-ico.bmp',
          height = 24,
          width = 23,
          color = { 0x40,0x00,0x80 },
          notifier = function()navt_last() end,
          tooltip = 'Last track/group\n[F12]'
        },
        vb:horizontal_aligner { width = 4 },
        vb:button {
          id = 'navs_up',
          bitmap = 'icons/up-ico.bmp',
          height = 24,
          width = 28,
          color = { 0x40,0x00,0x80 },
          --notifier = function()navs_up() end,
          pressed =  function()repeat_navs_up() end,
          released = function()repeat_navs_up(T) end,
          tooltip = 'Up sequence \n[]',
        },
        vb:button {
          id = 'navs_down',
          bitmap = 'icons/down-ico.bmp',
          height = 24,
          width = 28,
          color = { 0x40,0x00,0x80 },
          --notifier = function()navs_down() end,
          pressed =  function()repeat_navs_down() end,
          released = function()repeat_navs_down(T) end,
          tooltip = 'Down sequence\n[]'
        },
        vb:column {
          vb:button {
            id = 'navt_top',          
            bitmap = 'icons/top-ico.bmp',
            height = 12,
            width = 28,
            color = { 0x40,0x00,0x80 },
            notifier = function()navs_top() end,
            tooltip = 'Top pattern\n[]'
          },
          vb:button {
            id = 'navt_bottom',
            bitmap = 'icons/bottom-ico.bmp',
            height = 12,
            width = 28,
            color = { 0x40,0x00,0x80 },
            notifier = function()navs_bottom() end,
            tooltip = 'Bottom pattern\n[]'
          }
        }
      },
      -- NavTracks-------------------------------
      vb:column {
        id ='cbox_bar_f',
        visible = F,
      },  
      vb:column {
        id = 'cbox_bar_t',
        visible = F,
        vb:horizontal_aligner { height = 5 },
        vb:horizontal_aligner {
          mode = 'justify',
          ------------------------------------Nav Bar
          vb:horizontal_aligner {
            --width = 698,
            vb:text { text = 'Navigate Tracks ' },
            vb:slider {
              id = 'nav_track',
              min = 1,
              max = max_tracks(),
              value = value_tracks(),
              width = 580,
              notifier = function(val) 
                renoise.song().selected_track_index = math.floor(val)
              end
            },
            vb:text { id = 'index_value', text = '---' },
          },
          ------------------------------------ Nav bar
        }
      }
      ---
    },  
    --hlpmenu----------------------------------
    vb:column {
      visible = F,
      id = 'up_select_cls'
    },
    ----------------------------- Keys
    vb:column {
      vb:horizontal_aligner { height = 5 },
      visible = F,
      id = 'up_select_hlp',
      vb:column {
        style = 'group',
        margin = 5,
        vb:horizontal_aligner { 
          vb:text {
            font = 'bold',
            text = '☲K  Key Commands'
          }  
        },
        vb:horizontal_aligner { height = 5 },
        vb:column {
          style = 'body',
          width = 694,
          height = 24,
          margin = 8,
          txt_K00
        },
        vb:horizontal_aligner { height = 5 },    
        vb:horizontal_aligner {
          mode = "justify",
          vb:column {
            style = 'body',
            width = 206,
            height = '100%',
            margin = 8,
            txt_K01
          },
          vb:column {
            style = 'body',
            width = 226,
            height = '100%',
            margin = 8,
            txt_K02
          },
          vb:column {
            style = 'body',
            width = 252,
            height = '100%',
            margin = 8,
            txt_K03
          }
        }
      }
    },
    ------------------------------- About
    vb:column {
      vb:horizontal_aligner { height = 5 },
      visible = F,
      id = 'up_select_abt',
      vb:column {
        style = 'group',
        margin = 5,
        vb:horizontal_aligner {
          mode = 'justify',
          vb:column {
            vb:text {
              font = 'bold',
              text = '? About GT16-Colors   v1.2'
            }   
          },
          vb:column {
            vb:horizontal_aligner {
              but_cm.aq, but_cm.bk, but_cm.bl, but_cm.br, but_cm.gl, but_cm.gy, but_cm.gr, but_cm.og, but_cm.pl, but_cm.pn, but_cm.ps, but_cm.rd, but_cm.sk, but_cm.vl, but_cm.wh, but_cm.yl
            }
          }
        },
        vb:horizontal_aligner { height = 5 },
        vb:column {
          style = 'body',
          height = '100%',
          margin = 5,
          txt_A01,
          vb:horizontal_aligner {
            mode = 'right',
            txt_A02
          }
        }
      }
    },
    --------------------Preferences
    vb:column {
      id ='cbox_pref_f',
      visible = F,
    },  
    vb:column {
      id = 'cbox_pref_t',
      visible = F,
      vb:horizontal_aligner { height = 5 },
      vb:column {
        style = 'group',
        width = 707,
        margin = 5,
        vb: horizontal_aligner { mode = 'right',
          vb: horizontal_aligner { mode = 'center', width = 430, startup_prefs }, checkbox_autobar, autostart, 
        },
        vb:horizontal_aligner { height = 5 },
        vb:row {
          color_blend, note_col, template, sequence, midi
        },
        vb:row {
          bpm, lpb, no_lines, metronome, checkbox_autoplay, bt_apply_preferences,
        },
        vb:row {  }
      }
    }
    --------------------Preferences
  },
  ------------------------------------------------------------------------------ GT16-Colors
  ------------------------------------- Advanced Tools (Pattern Editor)
  vb:column {
    visible = F,
    id = 'up_select_tls',
    vb:horizontal_aligner { height = 5 },
    vb:column {  
      style = 'group',
      margin = 5,
      vb:horizontal_aligner {
        mode = 'right',
        vb:column {
          vb:horizontal_aligner {
            mode = 'right',
            vb:text {
              font = 'big',
              text = '⚒'
            },
            vb:text {
              font = 'bold',
              text = 'Advanced Tools '
            }
          }
        },
        vb:column {
        vb:horizontal_aligner {
          SWI_AT_SELECT,
          vb:horizontal_aligner { width = 4 },
          SWI_DW_SELECT
          }
        }
      },
      -------------------------------Tool 01
      vb:column {
        visible = F,
        id = 'at_select_01',
        style = 'body',
        width = 694,
        margin = 08,
        --- ---
        PEST, --Tool 01
        --- ---
      },
      -------------------------------Tool 02
      vb:column {
        visible = F,
        id = 'at_select_02',
        style = 'body',
        width = 694,
        margin = 08,
        --- ---
        PMSS, --Tool 02
        --- ---
      },
      -------------------------------Tool 03
      vb:column {
        visible = F,
        id = 'at_select_03',
        style = 'body',
        width = 694,
        margin = 08,
        --- ---        
        DCSP, --Tool 03
        --- ---        
      },
      -------------------------------Tool 04
      vb:column {
        visible = F,
        id = 'at_select_04',
        style = 'body',
        width = 694,
        margin = 08,
        --- ---
        GTC, --Tool 04
        --- ---
      },
      -------------------------------Tool 05
      vb:column {
        visible = F,
        id = 'at_select_05',
        style = 'body',
        width = 694,
        margin = 08,
        --- ---
        AMIC, --Tool 05
        --- ---
      },
      -------------------------------Tool 06
      vb:column {
        visible = F,
        id = 'at_select_06',
        style = 'body',
        width = 694,
        margin = 08,
        --- ---
        SPNO, --Tool 06
        --- ---
      },
      -------------------------------Tool 07
      vb:column {
        visible = F,
        id = 'at_select_07',
        style = 'body',
        width = 694,
        margin = 08,
        --- ---
        PETB, --Tool 07
        --- ---
      },
      -------------------------------Tool 08
      vb:column {
        visible = F,
        id = 'at_select_08',
        style = 'body',
        width = 694,
        margin = 08,
        --- ---
        WMP, --Tool 08
        --- ---
      },
      -------------------------------Tool 09
      vb:column {
        visible = F,
        id = 'at_select_09',
        style = 'body',
        width = 694,
        margin = 08,
        --- ---
        TL09, --Tool 09
        --- ---
      },
      -------------------------------Tool 10
      vb:column {
        visible = F,
        id = 'at_select_10',
        style = 'body',
        width = 694,
        margin = 08,
        --- ---
        TL10, --Tool 10
        --- ---
      }
      -------------------------------
    }
  }
}
-----------------------------------------------------
show_tab( 2 )    -- 2, start the things [1-6]
show_sgrtab( 1 ) -- 1, SubGroups/Octaves [1-4]
show_subtab( 1 ) -- 1, Combined [1-2]
---
up_select( prefs.up_selector.value ) -- 1, Hide/Keys/?/Advancet Tools [1-4]
dw_select( prefs.dw_selector.value )  -- 1, Show/hide Controls [1-2]
at_select( prefs.at_selector.value ) -- 4, Advanced Tools Selector 1-...-9-0 [10]
---
show_preferences( prefs.preferences.value ) -- Show/hide Preferences Pane [True/False]
show_cbox_bar( F ) -- F, Show/hide Navigation Bars [True/False]
-----------------------------------------------------


--------------------------------------------------------------------------------
-- SHOW_DIALOG
--------------------------------------------------------------------------------
function show_dialog()
  if dialog and dialog.visible then dialog:show() return end -- Prevents duplicate of the floating window ( when reaching this, the dialog never was created before or was closed ) 
  dialog = renoise.app():show_custom_dialog ( ' '.. tool_name .. '  v' .. tool_version .. '     by: ulneiz' , content , keyhandler )
  renoise.app():show_status('GT16-Colors is charged... Enjoy the colors and control!')
end


waiting_to_show_dialog = prefs.autostart.value

function idle_notifier_waiting()
  local idle_obs = renoise.tool().app_idle_observable
  local remove_notifier = F

  if waiting_to_show_dialog 
    and renoise.song() 
  then
    waiting_to_show_dialog = F
    show_dialog()
    --- ---
    if prefs.startup_prefs.value == F then
    else
      apply_preferences()
    end
    autobar()
    --- ---
    remove_notifier = T
  end  

  if remove_notifier and idle_obs:has_notifier( idle_notifier_waiting ) then
    idle_obs:remove_notifier( idle_notifier_waiting )
  end
end

if waiting_to_show_dialog then
  renoise.tool().app_idle_observable:add_notifier(idle_notifier_waiting)
end

-- Reload the script whenever this file is saved. Additionally, execute the attached function.
-- _AUTO_RELOAD_DEBUG = function() show_dialog() end
_AUTO_RELOAD_DEBUG = T



--------------------------------------------------------------------------------
-- MENU ENTRIES
--------------------------------------------------------------------------------
renoise.tool():add_menu_entry {
  name = 'Main Menu:Tools:'..tool_name..'... [* ALT + CTRL + G]',
  invoke = show_dialog
}

renoise.tool():add_menu_entry {
  name = 'Pattern Editor:'..tool_name..'... [* ALT + CTRL + G]',
  invoke = show_dialog
}

renoise.tool():add_menu_entry {
  name = 'Pattern Matrix:'..tool_name..'... [* ALT + CTRL + G]',
  invoke = show_dialog
}

renoise.tool():add_menu_entry {
  name = 'Mixer:'..tool_name..'... [* ALT + CTRL + G]',
  invoke = show_dialog
}

--------------------------------------------------------------------------------
-- KEY BINDING (RECOMENDED CTRL + ALT + G)
--------------------------------------------------------------------------------
renoise.tool():add_keybinding {
  name = 'Global:Tools:' ..tool_name..'... [* ALT + CTRL + G]',
  invoke = show_dialog
}

--------------------------------------------------------------------------------
-- MIDI/KEY MAPPING
--------------------------------------------------------------------------------
--[[
renoise.tool():add_midi_mapping {
  name = 'Tools:GT16-Colors:Clean All values in Track [Trigger]',                   -- MIDI MAPPING
  invoke = function()tool_01_33() end
}
]]

--spi, sti, sli, snci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index

--[[ UNICODE SYMBOLS

 ✓ ✔ ⇅ ⇆ ← → ↑ ↓ ↆ ❙❙❙ ☲ ❘❘❘ ☰ ⎓ ➤ ❤ ✦ ✉ ▬ ⚫ ◤ ◢ ◆ ┃ ℗ Ⓒ ➥ ✘ ✖ ⬒ ⬓ ▦ ⯬ ⇱ ☲K ? ⚒ ⚯ ☊ ☋ ♞ ●
 
 ≡◄ ⏴ ⏵ ▲ ▼  ◄ ◄◄ ►► ► ◂ ⏹ ⏯ ⏵⏸⏴ ⏪ ⏩ ⏮ ⏭ ☚ ☛ ↩ ↪ ↶ ↷ 🎹 ⚒ ☠ ☢ ☣ ⚠ ☺ ⧗ 🗗 🗕 ⇱ ⇲ ☊ ↹ ♬ ⭍ ♆

 http://unicode-table.com/en/#optical-character-recognition

--]]
