--------------------------------------------------------------------------------
-- Module Tool: Tool_08.lua
-- Name: WMP: Write Mouse Piano
-- Version: 1.0b
-- Author: ulneiz (Spain)
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- OBSERVABLE
--------------------------------------------------------------------------------
function focus_edit( song )
  song = renoise.song()
  if song.transport.edit_mode == false then
    vb.views['WMP_CTRL_FCS'].color = { 0x50,0x00,0x00 }
    else
    vb.views['WMP_CTRL_FCS'].color = { 0xF0,0x00,0x00 }
  end 
end
function focus_edit_mode( song )
  song = renoise.song()
  focus_edit()
  song.transport.edit_mode_observable:add_notifier( focus_edit )
end
renoise.tool().app_new_document_observable:add_notifier( focus_edit_mode ) -- document_observable function
--renoise.tool().app_new_document_observable:add_notifier( focus_edit )
--_AUTO_RELOAD_DEBUG = function() focus_edit() end


--------------------------------------------------------------------------------
-- OSC SERVER
--------------------------------------------------------------------------------
class 'OscClient'
 
function OscClient:__init( osc_host, osc_port, protocol )
  self._connection = nil
  local client, socket_error = renoise.Socket.create_client( osc_host, osc_port, protocol )
  if ( socket_error ) then 
    renoise.app():show_warning( "Warning: Failed to start the internal OSC client" )
    self._connection = nil
  else
    self._connection = client
  end
end

-- Trigger instrument-note
--- note_on (bool), true when note-on and false when note-off
--- instr    (int), the Renoise instrument index 1-254
--- track    (int), the Renoise track index 
--- note     (int), the desired pitch, 0-120
--- velocity (int), the desired velocity, 0-127
function OscClient:trigger_instrument( note_on, instr, track, note, velocity )
  if not self._connection then
    return false
  end
  local osc_vars = { }
    osc_vars[1] = { tag = "i", value = instr    }
    osc_vars[2] = { tag = "i", value = track    }
    osc_vars[3] = { tag = "i", value = note     }
  local header = nil
  if ( note_on ) then
    header = "/renoise/trigger/note_on"
    osc_vars[4] = { tag = "i", value = velocity }
  else
    header = "/renoise/trigger/note_off"
  end
  self._connection:send( renoise.Osc.Message( header,osc_vars ) )
  return true
end

osc_client = OscClient( "127.0.0.1", 8000, 2 )

--Instrument, track, note, vel
function wmp_instr( song, instr )
  song = renoise.song()
  instr = song.selected_instrument_index
  return instr
end

function wmp_track( song, track )
  song = renoise.song()
  track = song.selected_track_index
  return track
end

function wmp_note( note )
  note = vb.views['WMP_CTRL_NOT'].value
  return note
end

function wmp_vel( vel )
  vel = vb.views['WMP_CTRL_VEL'].value
  return vel
end


--Button Pressed for OSC Server
function wmp_fn_bt_pres( stored_edit_mode, song, spi, sti, sli, snci, column, patnte, patinst, patvol, timer_func )
  song = renoise.song()
  if song.transport.edit_mode == false then
    osc_client:trigger_instrument( true, wmp_instr(), wmp_track(), wmp_note(), wmp_vel() )
    return
  else
    spi, sti, sli, snci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index
    column = song:pattern(spi):track(sti):line(sli):note_column(snci)
    patnte, patinst, patvol = column.note_value, column.instrument_value, column.volume_value
    
    osc_client:trigger_instrument( true, wmp_instr(), wmp_track(), wmp_note(), wmp_vel() )

    timer_func = function()
      if ( column.note_value == wmp_note() and column.volume_value == wmp_vel() and column.instrument_value == wmp_instr() - 1 ) then
        renoise.tool():remove_timer( timer_func )
        column.note_value = patnte
        column.instrument_value = patinst
        column.volume_value = patvol
      end
    end
    renoise.tool():add_timer( timer_func, 20 ) --low is faster
  end
end
--Button Released for OSC Server
function wmp_fn_bt_rel()
  osc_client:trigger_instrument( false, wmp_instr(), wmp_track(), wmp_note(), wmp_vel() )
end


function repeat_key(release) --keys 0-119
  if not release then
    if renoise.tool():has_timer(repeat_key) then
      wmp_fn_bt_key()
      -- switch to fast timer on second run
      renoise.tool():remove_timer(repeat_key)
      if vb.views['WMP_CTRL_LGN'].value == 1 then
        renoise.tool():add_timer( wmp_fn_bt_key, 20 )
        else
        renoise.tool():add_timer( wmp_fn_bt_key, 40 )
      end
    else
      -- first run
      wmp_fn_bt_key()
      renoise.tool():add_timer( repeat_key, 400 )
    end
  else -- release all timers
    if renoise.tool():has_timer( repeat_key ) then
      renoise.tool():remove_timer( repeat_key )
    elseif renoise.tool():has_timer( wmp_fn_bt_key ) then
      renoise.tool():remove_timer( wmp_fn_bt_key )
    end
  end  
end

function wmp_fn_bt_key( song, spi, sti, sli, snci, vol, pan, dly, eff )
  song = renoise.song()
  if ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER and song.transport.edit_mode == true ) then  
    if ( song.selected_effect_column_index > 0 ) then
      song.selected_note_column_index = 1
    end
    spi, sti, sli, snci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index
    if vb.views['WMP_CTRL_NCTR'].value == true then
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ note_column_tracker() ].note_value = wmp_note() --note
      else
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].note_value = wmp_note() --note
    end
    ins_inst() --instrument
    --- ---
    if vb.views['WMP_CTRL_VAL'].value == true then
      vol = vb.views['WMP_CTRL_PP_VOL'].value --volume
      if ( song.tracks[ sti ].volume_column_visible == false and vol ~= 129 ) then
        song.tracks[ sti ].volume_column_visible = true
      end
      ins_vol()
      ---
      pan = vb.views['WMP_CTRL_PP_PAN'].value --pan
      if ( song.tracks[ sti ].panning_column_visible == false and pan ~= 129 ) then
        song.tracks[ sti ].panning_column_visible = true
      end
      ins_pan()
      ---
      dly = vb.views['WMP_CTRL_PP_DLY'].value --dly
      if ( song.tracks[ sti ].delay_column_visible == false and dly ~= 256 ) then
        song.tracks[ sti ].delay_column_visible = true
      end
      ins_dly()
      ---
      eff = vb.views['WMP_CTRL_PP_EFF'].value --eff, sample effects
      if ( song.tracks[ sti ].sample_effects_column_visible == false and eff > 1 and eff < 21 ) then
        song.tracks[ sti ].sample_effects_column_visible = true
      end
      if ( eff > 1 and eff < 21 ) then ins_sam_eff() ins_sam_amo() end
      ---      
    end
    step_length()    
  end
end


--------------------------------------------------------------------------------
-- CONTROLS (Functions and Elements)
--------------------------------------------------------------------------------

--FUNCTIONS
----Valuebox Step Length
function wmp_fn_vb_lng( song )
  song = renoise.song()
  song.transport.edit_step = vb.views['WMP_CTRL_LGN'].value
  vb.views['WMP_CTRL_LGN'].value = song.transport.edit_step
  if ( vb.views['WMP_CTRL_LGN'].value ~= 1 and vb.views['WMP_CTRL_NTR'].value == true ) then
    vb.views['WMP_CTRL_NTR'].value = false
    vb.views['WMP_CTRL_BT_NTR'].color = { 0x40,0x00,0x00 }
  end
end
function wmp_fn_vb_lng_value( song, step_length )
  song = renoise.song()
  step_length = song.transport.edit_step
  return step_length
end

function wmp_fn_vb( song )
  local function wmp_fn_vb_lng_obs( song, step_length )
    song = renoise.song()
    step_length = song.transport.edit_step
    vb.views['WMP_CTRL_LGN'].value = step_length
  end
  song = renoise.song()
  song.transport.edit_step_observable:add_notifier( wmp_fn_vb_lng_obs )
end
renoise.tool().app_new_document_observable:add_notifier( wmp_fn_vb )


----Instrument Rename
function wmp_fn_nme( song )
  song = renoise.song()
  local function wmp_fn_nme_obs( )
    vb.views['WMP_CTRL_NME'].text = ( "%s" ):format( song.selected_instrument.name )
    vb.views['WMP_CTRL_INS'].value = song.selected_instrument_index
  end
  song.selected_instrument_observable:add_notifier( wmp_fn_nme_obs )
end
renoise.tool().app_new_document_observable:add_notifier( function() if show_dialog then wmp_fn_nme() end end )  --< REVISAR SHOW_DIALOG PARA BARS

function wmp_rename_instr( song ) --rename instrument
  song = renoise.song()
  song.selected_instrument.name = vb.views['WMP_CTRL_NME'].text
end


function wmp_ctrl_ins( song, value ) --select instrument
  song = renoise.song()
  value = vb.views['WMP_CTRL_INS'].value 
  if (value  <= #song.instruments  ) then
    song.selected_instrument_index = value
  end
end


----Focus, Play, PlayZ and Stop
function wmp_fn_bt_fcs( song )
  song = renoise.song()
  if song.transport.edit_mode == false then
    song.transport.edit_mode = true
    else
    song.transport.edit_mode = false
  end
end

function wmp_fn_bt_ply( song, play_mode )
  song = renoise.song()
  play_mode = renoise.Transport.PLAYMODE_RESTART_PATTERN
  song.transport:start( play_mode )
  renoise.app():show_status( "GT16-Colors WMP:  song play." )
end

function wmp_fn_bt_plz( song, play_mode )
  song = renoise.song()
  play_mode = renoise.Transport.PLAYMODE_CONTINUE_PATTERN
  song.transport:start( play_mode )
  renoise.app():show_status( "GT16-Colors WMP:  play selected line." )
end

function wmp_fn_bt_stp( song )
  song = renoise.song()
  song.transport:stop()
  renoise.app():show_status( "GT16-Colors WMP:  song stop." )
end


----Volume, Panning, Delay Parameters | Clear Row, Default
function wmp_fn_pp_vol( value )
  value = vb.views['WMP_CTRL_PP_VOL'].value --[0-127, 255==Empty, 00-7F]
  if value == 129 then
    return 255
  else
    return (value - 1)
  end
end

function wmp_fn_pp_pan( value )
  value = vb.views['WMP_CTRL_PP_PAN'].value --[0-127, 255==Empty, 00-7F]
  if value == 129 then
    return 255
  else
    return (value - 1)
  end
end

function wmp_fn_pp_dly( value )
  value = vb.views['WMP_CTRL_PP_DLY'].value --[0-254, 255==Empty, 00-FE]
  if value == 256 then
    return 0
  else
    return value
  end
end


----Effect Parameters | For FXs and FXc
SELECT = "⚠ GT16-Colors WMP:  please, select before a value from the list to insert!!!" 

function wmp_fn_pp_eff( app, value )
  app = renoise.app()
  value = vb.views['WMP_CTRL_PP_EFF'].value --[00-ZZ]
  if        value ==  1 then app:show_status( SELECT ) return ''
    elseif  value ==  2 then return  '0A'
    elseif  value ==  3 then return  '0U'
    elseif  value ==  4 then return  '0D'
    elseif  value ==  5 then return  '0G'
    elseif  value ==  6 then return  '0I'
    elseif  value ==  7 then return  '0O'
    elseif  value ==  8 then return  '0C'
    elseif  value ==  9 then return  '0Q'
    elseif  value == 10 then return  '0M'
    elseif  value == 11 then return  '0S'
    elseif  value == 12 then return  '0B'
    elseif  value == 13 then return  '0R'
    elseif  value == 14 then return  '0Y'
    elseif  value == 15 then return  '0Z'
    elseif  value == 16 then return  '0V'
    elseif  value == 17 then return  '0T'
    elseif  value == 18 then return  '0N'
    elseif  value == 19 then return  '0E'
    elseif  value == 20 then return  ''
    --- --- 
    elseif  value == 21 then app:show_status( SELECT ) return ''
    elseif  value == 22 then return  '0A'
    elseif  value == 23 then return  '0U'
    elseif  value == 24 then return  '0D'
    elseif  value == 25 then return  '0G'
    elseif  value == 26 then return  '0I'
    elseif  value == 27 then return  '0O'
    elseif  value == 28 then return  '0C'
    elseif  value == 29 then return  '0Q'
    elseif  value == 30 then return  '0M'
    elseif  value == 31 then return  '0S'
    elseif  value == 32 then return  '0B'
    elseif  value == 33 then return  '0R'
    elseif  value == 34 then return  '0Y'
    elseif  value == 35 then return  '0Z'
    elseif  value == 36 then return  '0V'
    elseif  value == 37 then return  '0T'
    elseif  value == 38 then return  '0N'
    elseif  value == 39 then return  '0E'
    ---
    elseif  value == 40 then app:show_status( SELECT ) return ''
    elseif  value == 41 then return  '0L'
    elseif  value == 42 then return  '0P'
    elseif  value == 43 then return  '0W'
    elseif  value == 44 then return  '0J'
    elseif  value == 45 then return  '0X'
    ---
    elseif  value == 46 then app:show_status( SELECT ) return ''
    elseif  value == 47 then return  'ZT'
    elseif  value == 48 then return  'ZL'
    elseif  value == 49 then return  'ZK'
    elseif  value == 50 then return  'ZG'
    elseif  value == 51 then return  'ZB'
    elseif  value == 52 then return  'ZD'
    elseif  value == 53 then return  ''
  end
end


function wmp_fn_pp_amo( app, value )
  value = vb.views['WMP_CTRL_PP_AMO'].value --[0-127, 255==Empty, 00-7F]
  if value == 256 then
    return 0
  else
    return value
  end
end

--- ---

----Note Column Trarcker
function note_column_tracker( song )
  song = renoise.song()
  for i =   0, 11 do if vb.views['WMP_CTRL_NOT'].value == i then return i +   1 end end
  for i =  12, 23 do if vb.views['WMP_CTRL_NOT'].value == i then return i -  11 end end
  for i =  24, 35 do if vb.views['WMP_CTRL_NOT'].value == i then return i -  23 end end
  for i =  36, 47 do if vb.views['WMP_CTRL_NOT'].value == i then return i -  35 end end
  for i =  48, 59 do if vb.views['WMP_CTRL_NOT'].value == i then return i -  47 end end
  for i =  60, 71 do if vb.views['WMP_CTRL_NOT'].value == i then return i -  59 end end
  for i =  72, 83 do if vb.views['WMP_CTRL_NOT'].value == i then return i -  71 end end
  for i =  84, 95 do if vb.views['WMP_CTRL_NOT'].value == i then return i -  83 end end
  for i =  96,107 do if vb.views['WMP_CTRL_NOT'].value == i then return i -  95 end end
  for i = 108,119 do if vb.views['WMP_CTRL_NOT'].value == i then return i - 107 end end
end

----Insert Parameters
function ins_inst( song, spi, sti, sli, snci, sii ) --insert instrument value (note_columns)
  song = renoise.song()
  spi, sti, sli, snci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index
  sii = song.selected_instrument_index - 1 -- [01-FE, first is 01, not 00, then: -1]
  if vb.views['WMP_CTRL_NCTR'].value == true then
    song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ note_column_tracker() ].instrument_value = sii --instrument
  else
    song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci                  ].instrument_value = sii --instrument
  end
end

function ins_vol( song, spi, sti, sli, snci ) --insert volume value (note_columns)
  song = renoise.song()
  spi, sti, sli, snci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index
  if ( vb.views['WMP_CTRL_NTR'].value == true and vb.views['WMP_CTRL_LGN'].value == 1 and song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].note_value == 121 ) then --for Note Tracker
    else
    if vb.views['WMP_CTRL_NCTR'].value == true then
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ note_column_tracker() ].volume_value = wmp_fn_pp_vol()--[0-127, 255==Empty, 00-7F]
      else
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci                  ].volume_value = wmp_fn_pp_vol()--[0-127, 255==Empty, 00-7F]
    end
  end
end

function ins_pan( song, spi, sti, sli, snci ) --insert panning value (note_columns)
  song = renoise.song()
  spi, sti, sli, snci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index
  if ( vb.views['WMP_CTRL_NTR'].value == true and vb.views['WMP_CTRL_LGN'].value == 1 and song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].note_value == 121 ) then --for Note Tracker
    else
    if vb.views['WMP_CTRL_NCTR'].value == true then
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ note_column_tracker() ].panning_value = wmp_fn_pp_pan()--[0-127, 255==Empty, 00-7F]
      else
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci                  ].panning_value = wmp_fn_pp_pan()--[0-127, 255==Empty, 00-7F]
    end
  end    
end

function ins_dly( song, spi, sti, sli, snci ) --insert delay value (note_columns)
  song = renoise.song()
  spi, sti, sli, snci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index
  if ( vb.views['WMP_CTRL_NTR'].value == true and vb.views['WMP_CTRL_LGN'].value == 1 and song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].note_value == 121 ) then --for Note Tracker
    else
    if vb.views['WMP_CTRL_NCTR'].value == true then
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ note_column_tracker() ].delay_value = wmp_fn_pp_dly()--[0-255, .. ==Empty, 00-FF]
      else
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci                  ].delay_value = wmp_fn_pp_dly()--[0-255, .. ==Empty, 00-FF]
    end
  end
end  

function ins_sam_eff( song, spi, sti, sli, snci ) --insert effect FXs (note_columns)
  song = renoise.song()
  spi, sti, sli, snci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index
  if ( vb.views['WMP_CTRL_NTR'].value == true and vb.views['WMP_CTRL_LGN'].value == 1 and song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].note_value == 121 ) then --for Note Tracker
    else
    if vb.views['WMP_CTRL_NCTR'].value == true then
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ note_column_tracker() ].effect_number_string = wmp_fn_pp_eff()  --[string, '00' - 'ZZ'] number
      else
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci                  ].effect_number_string = wmp_fn_pp_eff()  --[string, '00' - 'ZZ'] number
    end
  end
end

function ins_sam_amo( song, spi, sti, sli, snci ) --insert amount FXs (note_columns)
  song = renoise.song()
  spi, sti, sli, snci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index
  if ( vb.views['WMP_CTRL_NTR'].value == true and vb.views['WMP_CTRL_LGN'].value == 1 and song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].note_value == 121 ) then --for Note Tracker
    else
    if vb.views['WMP_CTRL_NCTR'].value == true then
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ note_column_tracker() ].effect_amount_value = wmp_fn_pp_amo()  --[string, '00' - 'FF'] & amount
      else
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci                  ].effect_amount_value = wmp_fn_pp_amo()  --[string, '00' - 'FF'] & amount
    end
  end
end


function ins_col_eff( song, spi, sti, sli, seci ) --insert effect FXc (effect_columns)
  song = renoise.song()
  spi, sti, sli, seci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_effect_column_index
  song.patterns[ spi ].tracks[ sti ].lines[ sli ].effect_columns[ seci ].number_string = wmp_fn_pp_eff()  --[string, '00' - 'ZZ'] number
end

function ins_col_amo( song, spi, sti, sli, seci ) --insert amount FXc (effect_columns)
  song = renoise.song()
  spi, sti, sli, seci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_effect_column_index
  song.patterns[ spi ].tracks[ sti ].lines[ sli ].effect_columns[ seci ].amount_value = wmp_fn_pp_amo()  --[string, '00' - 'FF'] & amount
end

function step_length( song, spi, sti, sli, i, a, b, c ) --jump down line (even the next immediate pattern)
  song = renoise.song()
  spi, sti, sli = song.selected_pattern_index, song.selected_track_index, song.selected_line_index
  i = song.transport.edit_step                             --print('i: ', i)
  if ( sli + i <= #song.patterns[ spi ].tracks[ sti ].lines ) then
    song.selected_line_index = sli + i
    else 
    if ( song.selected_sequence_index ~= #song.sequencer.pattern_sequence ) then
      a = #song.patterns[ spi     ].tracks[ sti ].lines    --print('a: ', a)
      b = #song.patterns[ spi + 1 ].tracks[ sti ].lines    --print('b: ', b)
      c = a - song.selected_line_index                     --print('c: ', c)
      if ( a + b  >= song.selected_line_index + i ) then          
        song.selected_sequence_index = song.selected_sequence_index + 1
        song.selected_line_index = i - c
      end
      else 
        song.selected_line_index = #song.patterns[ spi ].tracks[ sti ].lines 
    end
  end
end

--- ---

function wmp_fn_bt_vol( song, sti, value )--Volume
  song = renoise.song()
  sti = song.selected_track_index
  value = vb.views['WMP_CTRL_PP_VOL'].value
  if ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER and song.transport.edit_mode == true ) then
    if ( song.tracks[ sti ].volume_column_visible == false and value ~= 129 ) then
      song.tracks[ sti ].volume_column_visible = true
    end
    if song.selected_effect_column_index > 0 then
      song.selected_note_column_index = 1
    end
    ins_vol()
    step_length()
  end
end

function wmp_fn_bt_pan( song, sti, value )--Panning
  song = renoise.song()
  sti = song.selected_track_index
  value = vb.views['WMP_CTRL_PP_PAN'].value
  if ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER and song.transport.edit_mode == true ) then
    if ( song.tracks[ sti ].panning_column_visible == false and value ~= 129 ) then
      song.tracks[ sti ].panning_column_visible = true
    end
    if song.selected_effect_column_index > 0 then
      song.selected_note_column_index = 1
    end
    ins_pan()
    step_length()
  end
end

function wmp_fn_bt_dly( song, sti, value )--Delay
  song = renoise.song()
  sti = song.selected_track_index
  value = vb.views['WMP_CTRL_PP_DLY'].value
  if ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER and song.transport.edit_mode == true ) then
    if ( song.tracks[ sti ].delay_column_visible == false and value ~= 256 ) then
      song.tracks[ sti ].delay_column_visible = true
    end
    if song.selected_effect_column_index > 0 then
      song.selected_note_column_index = 1
    end
    ins_dly()
    step_length()
  end
end

function wmp_fn_bt_eff( song, spi, sti, sli, snci, seci )--Effect FXs and FXc
  song = renoise.song()
  spi, sti, sli = song.selected_pattern_index, song.selected_track_index, song.selected_line_index
  if ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER and song.transport.edit_mode == true ) then
    if vb.views['WMP_CTRL_PP_EFF'].value > 1 and vb.views['WMP_CTRL_PP_EFF'].value < 21 then
      if song.tracks[ sti ].sample_effects_column_visible == false then
        song.tracks[ sti ].sample_effects_column_visible = true 
      end
      if song.selected_effect_column_index > 0 then
        song.selected_note_column_index = 1
      end
      ins_sam_eff()
      snci = song.selected_note_column_index
      if vb.views['WMP_CTRL_PP_AMO'].value < 257 then
        ins_sam_amo()
      end
      step_length()
    end
    --- ---
    if vb.views['WMP_CTRL_PP_EFF'].value > 21 and vb.views['WMP_CTRL_PP_EFF'].value < 40 then
      if song.tracks[ sti ].visible_effect_columns == 0 then
        song.tracks[ sti ].visible_effect_columns = song.selected_effect_column_index + 1
      end
      if song.selected_note_column_index > 0 then
        song.selected_effect_column_index = 1
      end
      ins_col_eff()
      seci = song.selected_effect_column_index
      if vb.views['WMP_CTRL_PP_AMO'].value < 257 then
        ins_col_amo()
      end
      step_length()
    end
    --- ---
    if vb.views['WMP_CTRL_PP_EFF'].value > 40 and vb.views['WMP_CTRL_PP_EFF'].value < 46 then
      if song.tracks[ sti ].visible_effect_columns == 0 then
        song.tracks[ sti ].visible_effect_columns = song.selected_effect_column_index + 1
      end
      if song.selected_note_column_index > 0 then
        song.selected_effect_column_index = 1
      end
      ins_col_eff()
      seci = song.selected_effect_column_index
      if vb.views['WMP_CTRL_PP_AMO'].value < 257 then
        ins_col_amo()
      end
      step_length()
    end
    --- ---
    if vb.views['WMP_CTRL_PP_EFF'].value > 46 and vb.views['WMP_CTRL_PP_EFF'].value < 54 then
      if song.tracks[ sti ].visible_effect_columns == 0 then
        song.tracks[ sti ].visible_effect_columns = song.selected_effect_column_index + 1
      end
      if song.selected_note_column_index > 0 then
        song.selected_effect_column_index = 1
      end
      ins_col_eff()
      seci = song.selected_effect_column_index
      if vb.views['WMP_CTRL_PP_AMO'].value < 257 then
        ins_col_amo()
      end
      step_length()
    end
  end
  --- --- ---  
  if ( song.selected_track.type ~= renoise.Track.TRACK_TYPE_SEQUENCER and song.transport.edit_mode == true ) then
    --- ---
    if vb.views['WMP_CTRL_PP_EFF'].value > 21 and vb.views['WMP_CTRL_PP_EFF'].value < 40 then
      ins_col_eff()
      seci = song.selected_effect_column_index
      if vb.views['WMP_CTRL_PP_AMO'].value < 257 then
        ins_col_amo()
      end
      step_length()
    end
    --- ---
    if vb.views['WMP_CTRL_PP_EFF'].value > 40 and vb.views['WMP_CTRL_PP_EFF'].value < 46 then
      ins_col_eff()
      seci = song.selected_effect_column_index
      if vb.views['WMP_CTRL_PP_AMO'].value < 257 then
        ins_col_amo()
      end
      step_length()
    end
    --- ---
    if vb.views['WMP_CTRL_PP_EFF'].value > 46 and vb.views['WMP_CTRL_PP_EFF'].value < 54 then
      ins_col_eff()
      seci = song.selected_effect_column_index
      if vb.views['WMP_CTRL_PP_AMO'].value < 257 then
        ins_col_amo()
      end
      step_length()
    end
  end  
end

--- ---

----Keys 120, 121, 122, clear ROW NC and clear ROW FXc
function repeat_key_120( release )
  if not release then
    if renoise.tool():has_timer( repeat_key_120 ) then
      wmp_fn_key_120()
      -- switch to fast timer on second run
      renoise.tool():remove_timer( repeat_key_120 )
      if vb.views['WMP_CTRL_LGN'].value == 1 then
        renoise.tool():add_timer( wmp_fn_key_120, 20 )
        else
        renoise.tool():add_timer( wmp_fn_key_120, 40 )
      end
      else
      -- first run
      wmp_fn_key_120()
      renoise.tool():add_timer( repeat_key_120, 400 )
    end
  else -- release all timers
    if renoise.tool():has_timer( repeat_key_120 ) then
      renoise.tool():remove_timer( repeat_key_120 )
    elseif renoise.tool():has_timer( wmp_fn_key_120 ) then
      renoise.tool():remove_timer( wmp_fn_key_120 )
    end
  end  
end
---
function repeat_key_121( release )
  if not release then
    if renoise.tool():has_timer( repeat_key_121 ) then
      wmp_fn_key_121()
      renoise.tool():remove_timer( repeat_key_121 )
      if vb.views['WMP_CTRL_LGN'].value == 1 then
        renoise.tool():add_timer( wmp_fn_key_121, 20 )
        else
        renoise.tool():add_timer( wmp_fn_key_121, 40 )
      end
    else
      wmp_fn_key_121()
      renoise.tool():add_timer( repeat_key_121, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_key_121 ) then
      renoise.tool():remove_timer( repeat_key_121 )
    elseif renoise.tool():has_timer( wmp_fn_key_121 ) then
      renoise.tool():remove_timer( wmp_fn_key_121 )
    end
  end  
end
---
function repeat_key_122( release )
  if not release then
    if renoise.tool():has_timer( repeat_key_122 ) then
      wmp_fn_key_122()
      renoise.tool():remove_timer( repeat_key_122 )
      if vb.views['WMP_CTRL_LGN'].value == 1 then
        renoise.tool():add_timer( wmp_fn_key_122, 20 )
      else
        renoise.tool():add_timer( wmp_fn_key_122, 40 )
      end
    else
      wmp_fn_key_122()
      renoise.tool():add_timer( repeat_key_122, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_key_122 ) then
      renoise.tool():remove_timer( repeat_key_122 )
    elseif renoise.tool():has_timer( wmp_fn_key_122 ) then
      renoise.tool():remove_timer( wmp_fn_key_122 )
    end
  end  
end
---
function repeat_clr_nc( release )
  if not release then
    if renoise.tool():has_timer( repeat_clr_nc ) then
      wmp_fn_bt_clr_nc()
      renoise.tool():remove_timer( repeat_clr_nc )
      if vb.views['WMP_CTRL_LGN'].value == 1 then
        renoise.tool():add_timer( wmp_fn_bt_clr_nc, 20 )
        else
        renoise.tool():add_timer( wmp_fn_bt_clr_nc, 40 )
      end
    else
      wmp_fn_bt_clr_nc()
      renoise.tool():add_timer( repeat_clr_nc, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_clr_nc ) then
      renoise.tool():remove_timer( repeat_clr_nc )
    elseif renoise.tool():has_timer( wmp_fn_bt_clr_nc ) then
      renoise.tool():remove_timer( wmp_fn_bt_clr_nc )
    end
  end  
end
---
function repeat_clr_fx( release )
  if not release then
    if renoise.tool():has_timer( repeat_clr_fx ) then
      wmp_fn_bt_clr_fx()
      renoise.tool():remove_timer( repeat_clr_fx )
      if vb.views['WMP_CTRL_LGN'].value == 1 then
        renoise.tool():add_timer( wmp_fn_bt_clr_fx, 20 )
        else
        renoise.tool():add_timer( wmp_fn_bt_clr_fx, 40 )
      end
    else
      wmp_fn_bt_clr_fx()
      renoise.tool():add_timer( repeat_clr_fx, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_clr_fx ) then
      renoise.tool():remove_timer( repeat_clr_fx )
    elseif renoise.tool():has_timer( wmp_fn_bt_clr_fx ) then
      renoise.tool():remove_timer( wmp_fn_bt_clr_fx )
    end
  end  
end
--- ---
function wmp_fn_key_120( song, spi, sti, sli, snci ) --Note-OFF
  song = renoise.song()
  if ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER and song.transport.edit_mode == true ) then
    if ( song.selected_effect_column_index > 0 ) then
      song.selected_note_column_index = 1
    end
    spi, sti, sli, snci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index
    song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].note_value = 120 --Note-OFF
    song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].instrument_value = 255
    step_length()
  end
end
---
function wmp_fn_key_121( song, spi, sti, sli, snci ) --Note Empty
  song = renoise.song()
  if ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER and song.transport.edit_mode == true ) then
    if ( song.selected_effect_column_index > 0 ) then
      song.selected_note_column_index = 1
    end
    spi, sti, sli, snci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index
    song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].note_value = 121 --Note Empty
    step_length()
  end
end
---
function wmp_fn_key_122( song, spi, sti, sli, snci, sii ) --Instrument Only
  song = renoise.song()
  if ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER and song.transport.edit_mode == true ) then
    if ( song.selected_effect_column_index > 0 ) then
      song.selected_note_column_index = 1
    end
    spi, sti, sli, snci, sii = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index, song.selected_instrument_index
    if ( vb.views['WMP_CTRL_NTR'].value == true and vb.views['WMP_CTRL_LGN'].value == 1 and song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].note_value >= 120 ) then --for Note Tracker (Note-OFF & Note Empty)
    else
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].instrument_value = sii - 1 --Instrument
    end
    step_length()
  end
end
---
function wmp_fn_bt_clr_nc( song, spi, sti, sli, snci ) --Clear ROW NC
  song = renoise.song()
  if ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER and song.transport.edit_mode == true ) then
    if ( song.selected_effect_column_index > 0 ) then
      song.selected_note_column_index = 1
    end
    spi, sti, sli, snci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index
    if vb.views['WMP_CTRL_NTR'].value == true then
      if song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].note_value >= 120 then --for Note Tracker (Note-OFF & Note Empty)
        song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].instrument_value = 255
        song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].volume_value = 255
        song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].panning_value = 255
        song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].delay_value = 0
        song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].effect_number_string = ''
        song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].effect_amount_string = ''
      end     
      else
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].note_value = 121
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].instrument_value = 255
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].volume_value = 255
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].panning_value = 255
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].delay_value = 0
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].effect_number_string = ''
      song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].effect_amount_string = ''
    end
    step_length()
  end
end
---
function wmp_fn_bt_clr_fx( song, spi, sti, sli, seci ) --Clear ROW FXc
  song = renoise.song()
  spi, sti, sli = song.selected_pattern_index, song.selected_track_index, song.selected_line_index
  if ( song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER and song.transport.edit_mode == true )  then
    if ( song.selected_note_column_index > 0 ) then
      song.selected_effect_column_index = 1
    end
    seci = song.selected_effect_column_index
    song.patterns[ spi ].tracks[ sti ].lines[ sli ].effect_columns[ seci ].number_string = ''
    song.patterns[ spi ].tracks[ sti ].lines[ sli ].effect_columns[ seci ].amount_string = ''
    step_length()
  end
  if ( song.selected_track.type ~= renoise.Track.TRACK_TYPE_SEQUENCER and song.transport.edit_mode == true )  then
    seci = song.selected_effect_column_index
    song.patterns[ spi ].tracks[ sti ].lines[ sli ].effect_columns[ seci ].number_string = ''
    song.patterns[ spi ].tracks[ sti ].lines[ sli ].effect_columns[ seci ].amount_string = ''
    step_length()
  end
end

--- ---

function wmp_fn_bt_def( song, sti, value )
  song = renoise.song()
  sti = song.selected_track_index
  vb.views['WMP_CTRL_VEL'].value = 80
  vb.views['WMP_CTRL_NCTR'].value = false --hidden
  vb.views['WMP_CTRL_BT_NCTR'].color = { 0x40,0x00,0x00 }
  vb.views['WMP_CTRL_VAL'].value = true --hidden
  vb.views['WMP_CTRL_BT_VAL'].color = { 0x00,0x70,0x00 }
  vb.views['WMP_CTRL_NTR'].value = false --hidden
  vb.views['WMP_CTRL_BT_NTR'].color = { 0x40,0x00,0x00 }
  vb.views['WMP_CTRL_LGN'].value = 1
  vb.views['WMP_CTRL_JMP'].value = 16
  vb.views['WMP_CTRL_PP_VOL'].value = 129
  vb.views['WMP_CTRL_PP_PAN'].value = 129
  vb.views['WMP_CTRL_PP_DLY'].value = 256
  vb.views['WMP_CTRL_PP_EFF'].value = 1
  vb.views['WMP_CTRL_PP_AMO'].value = 256
end


----Note Column Matrix (range 1-12)
function wmp_fn_bt_nc1( song )
  song = renoise.song()
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    song.selected_note_column_index = 1
  end
end
function wmp_fn_bt_nc2( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_note_columns < 2 then
      song.tracks[ sti ].visible_note_columns = 2
    end
  song.selected_note_column_index = 2
  end
end
function wmp_fn_bt_nc3( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_note_columns < 3 then
      song.tracks[ sti ].visible_note_columns = 3
    end
  song.selected_note_column_index = 3
  end
end
function wmp_fn_bt_nc4( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_note_columns < 4 then
      song.tracks[ sti ].visible_note_columns = 4
    end
  song.selected_note_column_index = 4
  end
end
function wmp_fn_bt_nc5( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_note_columns < 5 then
      song.tracks[ sti ].visible_note_columns = 5
    end
  song.selected_note_column_index = 5
  end
end
function wmp_fn_bt_nc6( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_note_columns < 6 then
      song.tracks[ sti ].visible_note_columns = 6
    end
  song.selected_note_column_index = 6
  end
end
function wmp_fn_bt_nc7( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_note_columns < 7 then
      song.tracks[ sti ].visible_note_columns = 7
    end
  song.selected_note_column_index = 7
  end
end
function wmp_fn_bt_nc8( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_note_columns < 8 then
      song.tracks[ sti ].visible_note_columns = 8
    end
  song.selected_note_column_index = 8
  end
end
function wmp_fn_bt_nc9( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_note_columns < 9 then
      song.tracks[ sti ].visible_note_columns = 9
    end
  song.selected_note_column_index = 9
  end
end
function wmp_fn_bt_nc10( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_note_columns < 10 then
      song.tracks[ sti ].visible_note_columns = 10
    end
  song.selected_note_column_index = 10
  end
end
function wmp_fn_bt_nc11( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_note_columns < 11 then
      song.tracks[ sti ].visible_note_columns = 11
    end
  song.selected_note_column_index = 11
  end
end
function wmp_fn_bt_nc12( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_note_columns < 12 then
      song.tracks[ sti ].visible_note_columns = 12
    end
  song.selected_note_column_index = 12
  end
end


----Directional Joystick
function repeat_trl( release )
  if not release then
    if renoise.tool():has_timer( repeat_trl ) then
      wmp_fn_bt_trl()
      -- switch to fast timer on second run
      renoise.tool():remove_timer( repeat_trl )
      renoise.tool():add_timer( wmp_fn_bt_trl, 40 )
    else
      -- first run
      wmp_fn_bt_trl()
      renoise.tool():add_timer( repeat_trl, 400 )
    end
  else -- release all timers
    if renoise.tool():has_timer( repeat_trl ) then
      renoise.tool():remove_timer( repeat_trl )
    elseif renoise.tool():has_timer( wmp_fn_bt_trl ) then
      renoise.tool():remove_timer( wmp_fn_bt_trl )
    end
  end  
end
---
function repeat_trr( release )
  if not release then
    if renoise.tool():has_timer( repeat_trr ) then
      wmp_fn_bt_trr()
      renoise.tool():remove_timer( repeat_trr )
      renoise.tool():add_timer( wmp_fn_bt_trr, 40 )
    else
      wmp_fn_bt_trr()
      renoise.tool():add_timer( repeat_trr, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_trr ) then
      renoise.tool():remove_timer( repeat_trr )
    elseif renoise.tool():has_timer( wmp_fn_bt_trr ) then
      renoise.tool():remove_timer( wmp_fn_bt_trr )
    end
  end  
end
---
function repeat_lft( release )
  if not release then
    if renoise.tool():has_timer( repeat_lft ) then
      wmp_fn_bt_lft()
      renoise.tool():remove_timer( repeat_lft )
      renoise.tool():add_timer( wmp_fn_bt_lft, 40 )
    else
      wmp_fn_bt_lft()
      renoise.tool():add_timer( repeat_lft, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_lft ) then
      renoise.tool():remove_timer( repeat_lft )
    elseif renoise.tool():has_timer( wmp_fn_bt_lft ) then
      renoise.tool():remove_timer( wmp_fn_bt_lft )
    end
  end  
end
---
function repeat_lf2( release )
  if not release then
    if renoise.tool():has_timer( repeat_lf2 ) then
      wmp_fn_bt_lf2()
      renoise.tool():remove_timer( repeat_lf2 )
      renoise.tool():add_timer( wmp_fn_bt_lf2, 60 )
    else
      wmp_fn_bt_lf2()
      renoise.tool():add_timer( repeat_lf2, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_lf2 ) then
      renoise.tool():remove_timer( repeat_lf2 )
    elseif renoise.tool():has_timer( wmp_fn_bt_lf2 ) then
      renoise.tool():remove_timer( wmp_fn_bt_lf2 )
    end
  end  
end
---
function repeat_rgt( release )
  if not release then
    if renoise.tool():has_timer( repeat_rgt ) then
      wmp_fn_bt_rgt()
      renoise.tool():remove_timer( repeat_rgt )
      renoise.tool():add_timer( wmp_fn_bt_rgt, 40 )
    else
      wmp_fn_bt_rgt()
      renoise.tool():add_timer( repeat_rgt, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_rgt ) then
      renoise.tool():remove_timer( repeat_rgt )
    elseif renoise.tool():has_timer( wmp_fn_bt_rgt ) then
      renoise.tool():remove_timer( wmp_fn_bt_rgt )
    end
  end  
end
---
function repeat_rg2( release )
  if not release then
    if renoise.tool():has_timer( repeat_rg2 ) then
      wmp_fn_bt_rg2()
      renoise.tool():remove_timer( repeat_rg2 )
      renoise.tool():add_timer( wmp_fn_bt_rg2, 60 )
    else
      wmp_fn_bt_rg2()
      renoise.tool():add_timer( repeat_rg2, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_rg2 ) then
      renoise.tool():remove_timer( repeat_rg2 )
    elseif renoise.tool():has_timer( wmp_fn_bt_rg2 ) then
      renoise.tool():remove_timer( wmp_fn_bt_rg2 )
    end
  end  
end
---
function repeat_up( release )
  if not release then
    if renoise.tool():has_timer( repeat_up ) then
      wmp_fn_bt_up()
      renoise.tool():remove_timer( repeat_up )
      renoise.tool():add_timer( wmp_fn_bt_up, 40 )
    else
      wmp_fn_bt_up()
      renoise.tool():add_timer( repeat_up, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_up ) then
      renoise.tool():remove_timer( repeat_up )
    elseif renoise.tool():has_timer( wmp_fn_bt_up ) then
      renoise.tool():remove_timer( wmp_fn_bt_up )
    end
  end  
end
---
function repeat_dwn( release )
  if not release then
    if renoise.tool():has_timer( repeat_dwn ) then
      wmp_fn_bt_dwn()
      renoise.tool():remove_timer( repeat_dwn )
      renoise.tool():add_timer( wmp_fn_bt_dwn, 40 )
    else
      wmp_fn_bt_dwn()
      renoise.tool():add_timer( repeat_dwn, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_dwn ) then
      renoise.tool():remove_timer( repeat_dwn )
    elseif renoise.tool():has_timer( wmp_fn_bt_dwn ) then
      renoise.tool():remove_timer( wmp_fn_bt_dwn )
    end
  end  
end
--- ---
function wmp_fn_bt_trl( song )
  song = renoise.song()
  if song.selected_track_index > 1 then
    song:select_previous_track()
  --renoise.app():show_status( "" )
  end  
end
---
function wmp_fn_bt_trr( song )
  song = renoise.song()
  if song.selected_track_index < song.sequencer_track_count then
    song:select_next_track()
  end
end
---
function wmp_fn_bt_lft( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.selected_effect_column_index >= 1 then
      song.selected_note_column_index = 1
    elseif song.selected_note_column_index > 1 then
      song.selected_note_column_index = song.selected_note_column_index - 1
    elseif song.selected_note_column_index == 1 then
      song.selected_note_column_index = song.tracks[ song.selected_track_index ].visible_note_columns -- .. = max visible note columns
    end
  end   
end
---
function wmp_fn_bt_lf2( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_effect_columns == 0 then
      song.tracks[ sti ].visible_effect_columns = song.selected_effect_column_index + 1
    end
    if song.selected_note_column_index > 0 then
      song.selected_effect_column_index = 1
    elseif ( song.tracks[ sti ].visible_effect_columns > 0 and song.selected_effect_column_index > 1 ) then
      song.selected_effect_column_index = song.selected_effect_column_index - 1
    elseif song.selected_effect_column_index == 1 then
      song.selected_effect_column_index = song.tracks[ sti ].visible_effect_columns -- .. = max visible effect columns
    end
  end
  if song.selected_track.type ~= renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.selected_note_column_index > 0 then
      song.selected_effect_column_index = 1
    elseif ( song.tracks[ sti ].visible_effect_columns > 0 and song.selected_effect_column_index > 1 ) then
      song.selected_effect_column_index = song.selected_effect_column_index - 1
    elseif song.selected_effect_column_index == 1 then
      song.selected_effect_column_index = song.tracks[ sti ].visible_effect_columns -- .. = max visible effect columns
    end  
  end
end
---
function wmp_fn_bt_rgt( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_note_columns > song.selected_note_column_index then
       song.selected_note_column_index = song.selected_note_column_index + 1
    elseif song.tracks[ sti ].visible_note_columns == song.selected_note_column_index and song.selected_note_column_index ~= 12 then
       song.tracks[ sti ].visible_note_columns = song.selected_note_column_index + 1
       song.selected_note_column_index = song.selected_note_column_index + 1
    elseif song.selected_note_column_index == 12 then
      song.selected_note_column_index = 1
    end
  end
end
---
function wmp_fn_bt_rg2( song, sti )
  song = renoise.song()
  sti = song.selected_track_index
  if song.selected_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_effect_columns == 0 then
      song.tracks[ sti ].visible_effect_columns = song.selected_effect_column_index + 1
    end
    if song.selected_note_column_index > 0 then
      song.selected_effect_column_index = 1
    elseif song.tracks[ sti ].visible_effect_columns > song.selected_effect_column_index then
      song.selected_effect_column_index = song.selected_effect_column_index + 1
    elseif song.tracks[ sti ].visible_effect_columns == song.selected_effect_column_index and song.selected_effect_column_index ~= 8 then
      song.tracks[ sti ].visible_effect_columns = song.selected_effect_column_index + 1
      song.selected_effect_column_index = song.selected_effect_column_index + 1
    elseif song.selected_effect_column_index == 8 then
      song.selected_effect_column_index = 1
    end
  end
  if song.selected_track.type ~= renoise.Track.TRACK_TYPE_SEQUENCER then
    if song.tracks[ sti ].visible_effect_columns > song.selected_effect_column_index then
      song.selected_effect_column_index = song.selected_effect_column_index + 1
    elseif song.tracks[ sti ].visible_effect_columns == song.selected_effect_column_index and song.selected_effect_column_index ~= 8 then
      song.tracks[ sti ].visible_effect_columns = song.selected_effect_column_index + 1
      song.selected_effect_column_index = song.selected_effect_column_index + 1
    elseif song.selected_effect_column_index == 8 then
      song.selected_effect_column_index = 1
    end
  end  
end
---
function wmp_fn_bt_up( song, spi, sti, sli, i, a, b, c ) --jump up line (even the previous immediate page) 
  song = renoise.song()
  spi, sti, sli = song.selected_pattern_index, song.selected_track_index, song.selected_line_index
  i = vb.views['WMP_CTRL_JMP'].value                       --print('i: ', i)
  if ( sli - i > 0 ) then
    song.selected_line_index = sli - i
    else
    if ( song.selected_sequence_index > 1 ) then
      a = #song.patterns[ spi     ].tracks[ sti ].lines    --print('a: ', a)
      b = #song.patterns[ spi - 1 ].tracks[ sti ].lines    --print('b: ', b)
      c = b - song.selected_line_index                     --print('c: ', c)
      --if ( a + b  >= song.selected_line_index + i ) then
      if ( b + song.selected_line_index > i ) then
        song.selected_sequence_index = song.selected_sequence_index - 1
          song.selected_line_index = song.selected_line_index - i + b
      end
      else
        song.selected_line_index = 1
    end
  end
end
---
function wmp_fn_bt_up2( song )
  song = renoise.song()
  song.selected_line_index = 1
end
---
function wmp_fn_bt_dwn( song, spi, sti, sli, i, a, b, c ) --jump down line (even the next immediate page)
  song = renoise.song()
  spi, sti, sli = song.selected_pattern_index, song.selected_track_index, song.selected_line_index
  i = vb.views['WMP_CTRL_JMP'].value                       --print('i: ', i)
  if ( sli + i <= #song.patterns[ spi ].tracks[ sti ].lines ) then
    song.selected_line_index = sli + i
    else 
    if ( song.selected_sequence_index ~= #song.sequencer.pattern_sequence ) then
      a = #song.patterns[ spi     ].tracks[ sti ].lines    --print('a: ', a)
      b = #song.patterns[ spi + 1 ].tracks[ sti ].lines    --print('b: ', b)
      c = a - song.selected_line_index                     --print('c: ', c)
      if ( a + b  >= song.selected_line_index + i ) then          
        song.selected_sequence_index = song.selected_sequence_index + 1
        song.selected_line_index = i - c
      end
      else 
        song.selected_line_index = #song.patterns[ spi ].tracks[ sti ].lines 
    end
  end
end
---
function wmp_fn_bt_dw2( song, spi, sti )
  song = renoise.song()
  spi, sti = song.selected_pattern_index, song.selected_track_index
  song.selected_line_index = #song.patterns[ spi ].tracks[ sti ].lines
end

--- ---

--ELEMENTS
WMP_CTRL_PP_VOL = vb:popup {
  id = 'WMP_CTRL_PP_VOL', width = 52, height = 20, value = 129, notifier = function() wmp_fn_pp_vol() end, tooltip = 'Volume Value.\n[00-7F; min = 00, max = 7F]\n.. = Empty.',
  items = { '  00','  01','  02','  03','  04','  05','  06','  07','  08','  09','  0A','  0B','  0C','  0D','  0E','  0F',
            '  10','  11','  12','  13','  14','  15','  16','  17','  18','  19','  1A','  1B','  1C','  1D','  1E','  1F',
            '  20','  21','  22','  23','  24','  25','  26','  27','  28','  29','  2A','  2B','  2C','  2D','  2E','  2F',
            '  30','  31','  32','  33','  34','  35','  36','  37','  38','  39','  3A','  3B','  3C','  3D','  3E','  3F',
            '  40','  41','  42','  43','  44','  45','  46','  47','  48','  49','  4A','  4B','  4C','  4D','  4E','  4F',
            '  50','  51','  52','  53','  54','  55','  56','  57','  58','  59','  5A','  5B','  5C','  5D','  5E','  5F',
            '  60','  61','  62','  63','  64','  65','  66','  67','  68','  69','  6A','  6B','  6C','  6D','  6E','  6F',
            '  70','  71','  72','  73','  74','  75','  76','  77','  78','  79','  7A','  7B','  7C','  7D','  7E','  7F','   . .'
  }
}


WMP_CTRL_PP_PAN = vb:popup {
  id = 'WMP_CTRL_PP_PAN', width = 52, height = 20, value = 129, notifier = function() wmp_fn_pp_pan() end, tooltip = 'Panning Value.\n[00-7F; L = 00, C = 40, R = FF]\n.. = Empty.',
  items = { '  00','  01','  02','  03','  04','  05','  06','  07','  08','  09','  0A','  0B','  0C','  0D','  0E','  0F',
            '  10','  11','  12','  13','  14','  15','  16','  17','  18','  19','  1A','  1B','  1C','  1D','  1E','  1F',
            '  20','  21','  22','  23','  24','  25','  26','  27','  28','  29','  2A','  2B','  2C','  2D','  2E','  2F',
            '  30','  31','  32','  33','  34','  35','  36','  37','  38','  39','  3A','  3B','  3C','  3D','  3E','  3F',
            '  40','  41','  42','  43','  44','  45','  46','  47','  48','  49','  4A','  4B','  4C','  4D','  4E','  4F',
            '  50','  51','  52','  53','  54','  55','  56','  57','  58','  59','  5A','  5B','  5C','  5D','  5E','  5F',
            '  60','  61','  62','  63','  64','  65','  66','  67','  68','  69','  6A','  6B','  6C','  6D','  6E','  6F',
            '  70','  71','  72','  73','  74','  75','  76','  77','  78','  79','  7A','  7B','  7C','  7D','  7E','  7F','   . .'
  }
}


WMP_CTRL_PP_DLY = vb:popup {
  id = 'WMP_CTRL_PP_DLY', width = 52, height = 20, value = 256, notifier = function() wmp_fn_pp_dly() end, tooltip = 'Delay Value.\n[01-FF; min = 01, max = FF]\n.. = Empty (00).',
  items = {        '  01','  02','  03','  04','  05','  06','  07','  08','  09','  0A','  0B','  0C','  0D','  0E','  0F',
            '  10','  11','  12','  13','  14','  15','  16','  17','  18','  19','  1A','  1B','  1C','  1D','  1E','  1F',
            '  20','  21','  22','  23','  24','  25','  26','  27','  28','  29','  2A','  2B','  2C','  2D','  2E','  2F',
            '  30','  31','  32','  33','  34','  35','  36','  37','  38','  39','  3A','  3B','  3C','  3D','  3E','  3F',
            '  40','  41','  42','  43','  44','  45','  46','  47','  48','  49','  4A','  4B','  4C','  4D','  4E','  4F',
            '  50','  51','  52','  53','  54','  55','  56','  57','  58','  59','  5A','  5B','  5C','  5D','  5E','  5F',
            '  60','  61','  62','  63','  64','  65','  66','  67','  68','  69','  6A','  6B','  6C','  6D','  6E','  6F',
            '  70','  71','  72','  73','  74','  75','  76','  77','  78','  79','  7A','  7B','  7C','  7D','  7E','  7F',
            '  80','  81','  82','  83','  84','  85','  86','  87','  88','  89','  8A','  8B','  8C','  8D','  8E','  8F',
            '  90','  91','  92','  93','  94','  95','  96','  97','  98','  99','  9A','  9B','  9C','  9D','  9E','  9F',
            '  A0','  A1','  A2','  A3','  A4','  A5','  A6','  A7','  A8','  A9','  AA','  AB','  AC','  AD','  AE','  AF',
            '  B0','  B1','  B2','  B3','  B4','  B5','  B6','  B7','  B8','  B9','  BA','  BB','  BC','  BD','  BE','  BF',
            '  C0','  C1','  C2','  C3','  C4','  C5','  C6','  C7','  C8','  C9','  CA','  CB','  CC','  CD','  CE','  CF',
            '  D0','  D1','  D2','  D3','  D4','  D5','  D6','  D7','  D8','  D9','  DA','  DB','  DC','  DD','  DE','  DF',
            '  E0','  E1','  E2','  E3','  E4','  E5','  E6','  E7','  E8','  E9','  EA','  EB','  EC','  ED','  EE','  EF',
            '  F0','  F1','  F2','  F3','  F4','  F5','  F6','  F7','  F8','  F9','  FA','  FB','  FC','  FD','  FE','  FF','   . .'
  }
}


WMP_CTRL_PP_EFF = vb:popup {
  id = 'WMP_CTRL_PP_EFF', width = 70, height = 20, value = 1, notifier = function() wmp_fn_pp_eff() end, tooltip = 'Effect Value, for FXs & FXc.\n.. = Empty.\nABxy: AB = Effect Value, xy = Amount Value.', items = { 
    ' FXs SM',
    '    ..A   : Arpeggio, x/y = first/second note offset in semitones ',
    '    ..U   : slide pitch Up by xx 1/16ths of a semitone  ',
    '    ..D   : slide pitch Down by xx 1/16ths of a semitone  ',
    '    ..G   : Glide towards given note by xx 1/16ths of a semitone  ',
    '    .. I    : fade volume In by xx volume units ',
    '    ..O   : fade volume Out by xx volume units  ',
    '    ..C   : Cut volume to x after y ticks ',
    '    ..Q   : delay playback of the line by xx ticks (00 - TPL) ', --
    '    ..M   : set channel volume level, 00 = -60dB, FF = +3dB ', --
    '    ..S   : trigger phrase from line xx ',
    '    ..B   : play sample Backwards (xx=00) or forwards (xx=01) ',
    '    ..R   : Retrigger instruments that are currently playing  ', --
    '    ..Y   : maYbe trigger the line with probability xx  ', --
    '    ..Z    : Trigger phrase number xx (01 - 7E)  ', --
    '    ..V   : set Vibrato, x = speed, y = depth ',
    '    ..T    : set Tremolo, x = speed, y = depth ',
    '    ..N   : set auto paN, x = speed, y = depth  ',
    '    ..E   : set the position of all active Envelope ',
    '     . .   : empty value for FXs SM (Sample Effects) ',
    --------------20 (1-20)
    ' FXc SM',
    '    ..A   : Arpeggio, x/y = first/second note offset in semitones ',
    '    ..U   : slide pitch Up by xx 1/16ths of a semitone  ',
    '    ..D   : slide pitch Down by xx 1/16ths of a semitone  ',
    '    ..G   : Glide towards given note by xx 1/16ths of a semitone  ',
    '    .. I    : fade volume In by xx volume units ',
    '    ..O   : fade volume Out by xx volume units  ',
    '    ..C   : Cut volume to x after y ticks ',
    '    ..Q   : delay playback of the line by xx ticks (00 - TPL) ', --
    '    ..M   : set channel volume level, 00 = -60dB, FF = +3dB ', --
    '    ..S   : trigger phrase from line xx ',
    '    ..B   : play sample Backwards (xx=00) or forwards (xx=01) ',
    '    ..R   : Retrigger instruments that are currently playing  ', --
    '    ..Y   : maYbe trigger the line with probability xx  ', --
    '    ..Z    : Trigger phrase number xx (01 - 7E)  ', --
    '    ..V   : set Vibrato, x = speed, y = depth ',
    '    ..T    : set Tremolo, x = speed, y = depth ',
    '    ..N   : set auto paN, x = speed, y = depth  ',
    '    ..E   : set the position of all active Envelope ',
    -------------19 (21-39)
    ' FXc TR',
    "    ..L   : set track pre-mixer's volume Level, 00=-INF, FF=+3dB ",
    "    ..P   : set track pre-mixer's Panning, 00=Lf, 80=Cn, FF =Rg  ",
    "    ..W   : set track pre-mixer's surround width, 00=off, FF=max.  ",
    "    ..J    : set track's output routing to channel xx ",
    '    ..X   : stop all notes and FX, X00 or specific effect, xx>00 ',
    -------------07 (40-46)
    ' FXc GL',
    '     ZT   : set Tempo (BPM) (20 - FF, 00 = stop song)  ',
    '     ZL   : set Lines Per Beat (LPB) (01 - FF, 00 = stop song) ',
    '     ZK   : set Ticks Per Line (TPL) (01 - 10) ',
    '     ZG   : toggle song Groove on/off (00 = off, >00 = on) ',
    '     ZB   : Break pattern. Finishes & jumps next pattern (line xx) ',
    '     ZD   : Delay (pause) pattern playback by xx lines ',
    '      . .   : empty value for FXc SM, FXc TR/GB (Track/Global FX) ',
    -------------08 (47-54)
  }
}


WMP_CTRL_PP_AMO = vb:popup {
  id = 'WMP_CTRL_PP_AMO', width = 52, height = 20, value = 256, notifier = function() wmp_fn_pp_amo() end, tooltip = 'Amount Value (..xy).\n[00-FF; min = 00, max = FF].\nNeed the Effect Value.',
  items = {        '  01','  02','  03','  04','  05','  06','  07','  08','  09','  0A','  0B','  0C','  0D','  0E','  0F',
            '  10','  11','  12','  13','  14','  15','  16','  17','  18','  19','  1A','  1B','  1C','  1D','  1E','  1F',
            '  20','  21','  22','  23','  24','  25','  26','  27','  28','  29','  2A','  2B','  2C','  2D','  2E','  2F',
            '  30','  31','  32','  33','  34','  35','  36','  37','  38','  39','  3A','  3B','  3C','  3D','  3E','  3F',
            '  40','  41','  42','  43','  44','  45','  46','  47','  48','  49','  4A','  4B','  4C','  4D','  4E','  4F',
            '  50','  51','  52','  53','  54','  55','  56','  57','  58','  59','  5A','  5B','  5C','  5D','  5E','  5F',
            '  60','  61','  62','  63','  64','  65','  66','  67','  68','  69','  6A','  6B','  6C','  6D','  6E','  6F',
            '  70','  71','  72','  73','  74','  75','  76','  77','  78','  79','  7A','  7B','  7C','  7D','  7E','  7F',
            '  80','  81','  82','  83','  84','  85','  86','  87','  88','  89','  8A','  8B','  8C','  8D','  8E','  8F',
            '  90','  91','  92','  93','  94','  95','  96','  97','  98','  99','  9A','  9B','  9C','  9D','  9E','  9F',
            '  A0','  A1','  A2','  A3','  A4','  A5','  A6','  A7','  A8','  A9','  AA','  AB','  AC','  AD','  AE','  AF',
            '  B0','  B1','  B2','  B3','  B4','  B5','  B6','  B7','  B8','  B9','  BA','  BB','  BC','  BD','  BE','  BF',
            '  C0','  C1','  C2','  C3','  C4','  C5','  C6','  C7','  C8','  C9','  CA','  CB','  CC','  CD','  CE','  CF',
            '  D0','  D1','  D2','  D3','  D4','  D5','  D6','  D7','  D8','  D9','  DA','  DB','  DC','  DD','  DE','  DF',
            '  E0','  E1','  E2','  E3','  E4','  E5','  E6','  E7','  E8','  E9','  EA','  EB','  EC','  ED','  EE','  EF',
            '  F0','  F1','  F2','  F3','  F4','  F5','  F6','  F7','  F8','  F9','  FA','  FB','  FC','  FD','  FE','  FF','  00'  
  }
}
-----------------------------------------------------
function repeat_volume( release )
  if not release then
    if renoise.tool():has_timer( repeat_volume ) then
      wmp_fn_bt_vol()
      -- switch to fast timer on second run
      renoise.tool():remove_timer( repeat_volume )
      if vb.views['WMP_CTRL_LGN'].value == 1 then
        renoise.tool():add_timer( wmp_fn_bt_vol, 20 )
        else
        renoise.tool():add_timer( wmp_fn_bt_vol, 40 )
      end
    else
      -- first run
      wmp_fn_bt_vol()
      renoise.tool():add_timer( repeat_volume, 400 )
    end
  else -- release all timers
    if renoise.tool():has_timer( repeat_volume ) then
      renoise.tool():remove_timer( repeat_volume )
    elseif renoise.tool():has_timer( wmp_fn_bt_vol ) then
      renoise.tool():remove_timer( wmp_fn_bt_vol )
    end
  end  
end
--- ---
function repeat_panning( release )
  if not release then
    if renoise.tool():has_timer( repeat_panning ) then
      wmp_fn_bt_pan()
      renoise.tool():remove_timer( repeat_panning )
      if vb.views['WMP_CTRL_LGN'].value == 1 then
        renoise.tool():add_timer( wmp_fn_bt_pan, 20 )
        else
        renoise.tool():add_timer( wmp_fn_bt_pan, 40 )
      end
    else
      wmp_fn_bt_pan()
      renoise.tool():add_timer( repeat_panning, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_panning ) then
      renoise.tool():remove_timer( repeat_panning )
    elseif renoise.tool():has_timer( wmp_fn_bt_pan ) then
      renoise.tool():remove_timer( wmp_fn_bt_pan )
    end
  end  
end
--- ---
function repeat_delay( release )
  if not release then
    if renoise.tool():has_timer( repeat_delay ) then
      wmp_fn_bt_dly()
      renoise.tool():remove_timer( repeat_delay )
      if vb.views['WMP_CTRL_LGN'].value == 1 then
        renoise.tool():add_timer( wmp_fn_bt_dly, 20 )
        else
        renoise.tool():add_timer( wmp_fn_bt_dly, 40 )
      end
    else
      wmp_fn_bt_dly()
      renoise.tool():add_timer( repeat_delay, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_delay ) then
      renoise.tool():remove_timer( repeat_delay )
    elseif renoise.tool():has_timer( wmp_fn_bt_dly ) then
      renoise.tool():remove_timer( wmp_fn_bt_dly )
    end
  end  
end
--- ---
function repeat_effect( release )
  if not release then
    if renoise.tool():has_timer( repeat_effect ) then
      wmp_fn_bt_eff()
      renoise.tool():remove_timer( repeat_effect )
      if vb.views['WMP_CTRL_LGN'].value == 1 then
        renoise.tool():add_timer( wmp_fn_bt_eff, 20 )
        else
        renoise.tool():add_timer( wmp_fn_bt_eff, 40 )
      end
    else
      wmp_fn_bt_eff()
      renoise.tool():add_timer( repeat_effect, 400 )
    end
  else
    if renoise.tool():has_timer( repeat_effect ) then
      renoise.tool():remove_timer( repeat_effect )
    elseif renoise.tool():has_timer( wmp_fn_bt_eff ) then
      renoise.tool():remove_timer( wmp_fn_bt_eff )
    end
  end  
end
-----------------------------------------------------
WMP_CTRL_BT_VOL = vb:button {
  id = 'WMP_CTRL_BT_VOL',
  tooltip = 'Volume Value\n(Work also with NCTr or NTr).',
  text = 'VOL',
  width = 52,
  height = 20,
  color = { 0xBC,0xB5,0x25 },
  --notifier = function() wmp_fn_bt_vol() end,
  pressed  = function() repeat_volume() end,
  released = function() repeat_volume(true) end
}

WMP_CTRL_BT_PAN = vb:button {
  id = 'WMP_CTRL_BT_PAN',
  tooltip = 'Panning Value\n(Work also with NCTr or NTr).',
  text = 'PAN',
  width = 52,
  height = 20,
  color = { 0x8E,0xB7,0x7E },
  --notifier = function() wmp_fn_bt_pan() end,
  pressed  = function() repeat_panning() end,
  released = function() repeat_panning(true) end
}

WMP_CTRL_BT_DLY = vb:button {
  id = 'WMP_CTRL_BT_DLY',
  tooltip = 'Delay Value\n(Work also with NCTr or NTr).',
  text = 'DLY',
  width = 52,
  height = 20,
  color = { 0x36,0xA1,0xC1 },
  --notifier = function() wmp_fn_bt_dly() end,
  pressed  = function() repeat_delay() end,
  released = function() repeat_delay(true) end
}

WMP_CTRL_BT_EFF = vb:button {
  id = 'WMP_CTRL_BT_EFF',
  tooltip = 'Effect Value\n(Only FXs work also with NCTr or NTr).',
  text = 'FX',
  width = 34,
  height = 20,
  color = { 0x9E,0x44,0x1E },
  --notifier = function() wmp_fn_bt_eff() end,
  pressed  = function() repeat_effect() end,
  released = function() repeat_effect(true) end
}

--- --- ---

WMP_CTRL_CLR_NC = vb:button {
  id = 'WMP_CTRL_CLR_NC',
  tooltip = "Clear Row in Note Column\n[Note, Instrument, Vol, Pan, Dly & FXs].\nor\nIf 'NTr' is activted then: Clear Row in Note Column for empty notes only (preserve Note-OFFs).",
  text = 'RW NC',
  width = 45,
  height = 22,
  color = { 0x70,0x00,0x00 },
  --notifier = function() wmp_fn_bt_clr_nc() end,
  pressed  = function() repeat_clr_nc() end,
  released = function() repeat_clr_nc(true) end
}

WMP_CTRL_CLR_FX = vb:button {
  id = 'WMP_CTRL_CLR_FX',
  text = 'RW FX',
  tooltip = 'Clear Row in Effect Column (FXc)\n[All, Effect Values and Amount Values].',
  width = 45,
  height = 22,
  color = { 0x70,0x00,0x00 },
  --notifier = function() wmp_fn_bt_clr_fx() end,
  pressed  = function() repeat_clr_fx() end,
  released = function() repeat_clr_fx(true) end
}

--Hidden Checkboxes 'Note Column Tracker', 'Write Values' & 'Note Tracker'
WMP_CTRL_NCTR = vb:row { vb:text { text = 'NC' },
  vb:checkbox { id = 'WMP_CTRL_NCTR', value = false,
  notifier = function(value) if value == true then vb.views['WMP_CTRL_NTR'].value = false vb.views['WMP_CTRL_BT_NTR'].color = { 0x40,0x00,0x00 } end end }
}
--- ---
WMP_CTRL_VAL = vb:row { vb:text { text = ' Val' },
  vb:checkbox { id = 'WMP_CTRL_VAL', value = true }
}
--- ---
WMP_CTRL_NTR = vb:row { vb:text { text = ' NTr' },
  vb:checkbox { id = 'WMP_CTRL_NTR', value = false,
  notifier = function(value) if value == true then vb.views['WMP_CTRL_NCTR'].value = false vb.views['WMP_CTRL_BT_NCTR'].color = { 0x40,0x00,0x00 } vb.views['WMP_CTRL_LGN'].value = 1 end end }
}

--Visible Buttons
WMP_CTRL_BT_NCTR = vb:button {
  id = 'WMP_CTRL_BT_NCTR',
  width = 40,
  height = 22,
  text = 'NCTr',
  color = { 0x40,0x00,0x00 },
  notifier = function()
    if vb.views['WMP_CTRL_NCTR'].value == false then
       vb.views['WMP_CTRL_NCTR'].value = true
       vb.views['WMP_CTRL_BT_NCTR'].color = { 0x00,0x70,0x00 }
      else
       vb.views['WMP_CTRL_NCTR'].value = false
       vb.views['WMP_CTRL_BT_NCTR'].color = { 0x40,0x00,0x00 }
    end
  end,
  tooltip = "Activate 'Note Column Tracker' (NCTr).\nDistribute the notes according to the note columns."
}
--- ---
WMP_CTRL_BT_VAL = vb:button {
  id = 'WMP_CTRL_BT_VAL',
  width = 40,
  height = 22,
  text = 'Val',
  color = { 0x00,0x70,0x00 },
  notifier = function()
    if vb.views['WMP_CTRL_VAL'].value == false then
       vb.views['WMP_CTRL_VAL'].value = true
       vb.views['WMP_CTRL_BT_VAL'].color = { 0x00,0x70,0x00 }
      else
       vb.views['WMP_CTRL_VAL'].value = false
       vb.views['WMP_CTRL_BT_VAL'].color = { 0x40,0x00,0x00 }
    end
  end,
  tooltip = 'Activate write of values for keys.\n[Vol, Pan, Dly & FXs]. Not for FXc!'
}
--- ---
WMP_CTRL_BT_NTR = vb:button {
  id = 'WMP_CTRL_BT_NTR',
  width = 40,
  height = 22,
  text = 'NTr',
  color = { 0x40,0x00,0x00 },
  notifier = function()
    if vb.views['WMP_CTRL_NTR'].value == false then
       vb.views['WMP_CTRL_NTR'].value = true
       vb.views['WMP_CTRL_BT_NTR'].color = { 0x00,0x70,0x00 }
      else
       vb.views['WMP_CTRL_NTR'].value = false
       vb.views['WMP_CTRL_BT_NTR'].color = { 0x40,0x00,0x00 }
    end
  end,
  tooltip = "Activate 'Note Tracker' (NTr) for Step Length (Lgt) = 01.\n[INS (Instrument Key), Vol, Pan, Dly, FXs]. Not for FXc!\nOnly write these values if exist the Note (or Note-OFF, except INS).\nWork also with 'RW NC'."
}

--- --- ---

WMP_CTRL_DEF = vb:button { id = 'WMP_CTRL_DEF', width = 30, height = 22, color = { 0x00,0x20,0x50 }, text = 'DF', notifier = function() wmp_fn_bt_def() end,
  tooltip = 'Default Values.\nLgt [01], Vol [..], Pan [..], Dly [..].\nFX [FXs SM], Amo [00].\nJump [16].\nValues (Val) = [enabled].\nNote Tracker (NTr) = [disabled].'
}

--- --- ---

WMP_CTRL_JMP = vb:valuebox { id = 'WMP_CTRL_JMP', width = 52, height = 20, min = 0, max = 128, value = 16,
  tooltip = "Jump 'n' lines.\n[n, 0-128; default = 16].\nIt prevents jumping a entire consecutive pattern!"
}

WMP_CTRL_TRL = vb:button { id = 'WMP_CTRL_TRL', width = 40, height = 20, color = { 0x4F,0x0C,0x89 }, bitmap = 'icons/left2-ico.bmp',
  pressed = function() repeat_trl() end, released = function() repeat_trl(true) end, tooltip = 'Previous Track.'
}

WMP_CTRL_TRR = vb:button { id = 'WMP_CTRL_TRR', width = 40, height = 20, color = { 0x4F,0x0C,0x89 }, bitmap = 'icons/right2-ico.bmp',
  pressed = function() repeat_trr() end, released = function() repeat_trr(true) end, tooltip = 'Next Track.'
}
--- ---
WMP_CTRL_LFT = vb:button { id = 'WMP_CTRL_LFT', width = 26, height = 38, color = { 0x81,0x10,0x59 }, bitmap = 'icons/left-ico.bmp',
  pressed = function() repeat_lft() end, released = function() repeat_lft(true) end, tooltip = 'Previous Note Column.'
}

WMP_CTRL_LF2 = vb:button { id = 'WMP_CTRL_LF2', width = 14, height = 38, color = { 0x81,0x10,0x59 }, bitmap = 'icons/left-fx-ico.bmp',
  pressed = function() repeat_lf2() end, released = function() repeat_lf2(true) end, tooltip = 'Previous FX Column.'
}

WMP_CTRL_RGT = vb:button { id = 'WMP_CTRL_RGT', width = 26, height = 38, color = { 0x81,0x10,0x59 }, bitmap = 'icons/right-ico.bmp',
  pressed = function() repeat_rgt() end, released = function() repeat_rgt(true) end, tooltip = 'Next Note Column.'
}

WMP_CTRL_RG2 = vb:button { id = 'WMP_CTRL_RG2', width = 14, height = 38, color = { 0x81,0x10,0x59 }, bitmap = 'icons/right-fx-ico.bmp',
  pressed = function() repeat_rg2() end, released = function() repeat_rg2(true) end, tooltip = 'Next FX Column.'
}

WMP_CTRL_UP  = vb:button { id = 'WMP_CTRL_UP',  width = 36, height = 19, color = { 0x81,0x10,0x59 }, bitmap = 'icons/up2-ico.bmp',
  pressed = function() repeat_up()  end, released = function() repeat_up(true) end, tooltip = 'Up Row.'
}

WMP_CTRL_UP2 = vb:button { id = 'WMP_CTRL_UP2', width = 16, height = 19, color = { 0x81,0x10,0x59 }, bitmap = 'icons/top2-ico.bmp',
  notifier = function() wmp_fn_bt_up2() end, tooltip = 'First Row.'
}

WMP_CTRL_DWN = vb:button { id = 'WMP_CTRL_DWN', width = 36, height = 19, color = { 0x81,0x10,0x59 }, bitmap = 'icons/down2-ico.bmp',
  pressed = function() repeat_dwn() end, released = function() repeat_dwn(true) end, tooltip = 'Down Row.'
}

WMP_CTRL_DW2 = vb:button { id = 'WMP_CTRL_DW2', width = 16, height = 19, color = { 0x81,0x10,0x59 }, bitmap = 'icons/bottom2-ico.bmp',
  notifier = function() wmp_fn_bt_dw2() end, tooltip = 'Last Row.'
}

--- --- ---
WMP_CTRL_INS = vb:popup {
  id = 'WMP_CTRL_INS', width = 58, height = 22, value = 1, notifier = function() wmp_ctrl_ins() end, tooltip = "Navigate for Instruments.\nUse also with 'INS' key.",
  items = { '  00','  01','  02','  03','  04','  05','  06','  07','  08','  09','  0A','  0B','  0C','  0D','  0E','  0F',
            '  10','  11','  12','  13','  14','  15','  16','  17','  18','  19','  1A','  1B','  1C','  1D','  1E','  1F',
            '  20','  21','  22','  23','  24','  25','  26','  27','  28','  29','  2A','  2B','  2C','  2D','  2E','  2F',
            '  30','  31','  32','  33','  34','  35','  36','  37','  38','  39','  3A','  3B','  3C','  3D','  3E','  3F',
            '  40','  41','  42','  43','  44','  45','  46','  47','  48','  49','  4A','  4B','  4C','  4D','  4E','  4F',
            '  50','  51','  52','  53','  54','  55','  56','  57','  58','  59','  5A','  5B','  5C','  5D','  5E','  5F',
            '  60','  61','  62','  63','  64','  65','  66','  67','  68','  69','  6A','  6B','  6C','  6D','  6E','  6F',
            '  70','  71','  72','  73','  74','  75','  76','  77','  78','  79','  7A','  7B','  7C','  7D','  7E','  7F',
            '  80','  81','  82','  83','  84','  85','  86','  87','  88','  89','  8A','  8B','  8C','  8D','  8E','  8F',
            '  90','  91','  92','  93','  94','  95','  96','  97','  98','  99','  9A','  9B','  9C','  9D','  9E','  9F',
            '  A0','  A1','  A2','  A3','  A4','  A5','  A6','  A7','  A8','  A9','  AA','  AB','  AC','  AD','  AE','  AF',
            '  B0','  B1','  B2','  B3','  B4','  B5','  B6','  B7','  B8','  B9','  BA','  BB','  BC','  BD','  BE','  BF',
            '  C0','  C1','  C2','  C3','  C4','  C5','  C6','  C7','  C8','  C9','  CA','  CB','  CC','  CD','  CE','  CF',
            '  D0','  D1','  D2','  D3','  D4','  D5','  D6','  D7','  D8','  D9','  DA','  DB','  DC','  DD','  DE','  DF',
            '  E0','  E1','  E2','  E3','  E4','  E5','  E6','  E7','  E8','  E9','  EA','  EB','  EC','  ED','  EE','  EF',
            '  F0','  F1','  F2','  F3','  F4','  F5','  F6','  F7','  F8','  F9','  FA','  FB','  FC','  FD','  FE',
  }
}


WMP_CTRL_NME = vb:column {
  vb:textfield { id = 'WMP_CTRL_NME', align = 'center', text = "Name of Instrument.", width = 200, height = 24, notifier = function () wmp_rename_instr() end, tooltip = 'Rename Instrument.' }
}

--- --- ---

WMP_CTRL_FCS = vb:button { id = 'WMP_CTRL_FCS', width = 29, height = 22, color = { 0x50,0x00,0x00 }, bitmap = 'icons/edit-mode-ico.bmp', notifier = function() wmp_fn_bt_fcs() end, tooltip = 'Edit Mode in Pattern Editor.\n[ESC]' }

WMP_CTRL_PLY = vb:button { id = 'WMP_CTRL_PLY', width = 20, height = 22, color = { 0xC0,0x50,0x00 }, bitmap = 'icons/play-ico.bmp', notifier = function() wmp_fn_bt_ply() end, tooltip = 'Play Song from initial line.' }

WMP_CTRL_PLZ = vb:button { id = 'WMP_CTRL_PLZ', width = 10, height = 22, color = { 0xC0,0x50,0x00 }, notifier = function() wmp_fn_bt_plz() end, tooltip = 'Play Song from position.' }

WMP_CTRL_STP = vb:button { id = 'WMP_CTRL_STP', width = 29, height = 22, color = { 0xC0,0x50,0x00 }, bitmap = 'icons/stop-ico.bmp', notifier = function() wmp_fn_bt_stp() end, tooltip = 'Stop Song.' }

WMP_CTRL_VEL = vb:row { vb:bitmap { bitmap = 'icons/volume-ico.bmp', mode = 'body_color', height = 22 }, vb:valuebox {
  id = 'WMP_CTRL_VEL', width = 52, height = 22, min = 0, max = 127, value = 80,
  tooltip = 'Volume Mouse Piano.\n[0-127]; default = 80.\nNo Sound? Activate OSC Server in Preferences [127.0.0.1, Port:8000, Protocol:UDP].' } --[127.0.0.1, 8000, 2]
} 

--- --- ---

WMP_CTRL_LGN = vb:row { vb:bitmap { bitmap = 'icons/step-length-ico.bmp', mode = 'body_color', height = 20, width = 18, tooltip = 'Step Length (Lgt)' },
  vb:valuebox { id = 'WMP_CTRL_LGN', width = 48, height = 20, min = 0, max = 64, value = 1, notifier = function() wmp_fn_vb_lng() end,
    tooltip = "Step Length for Pattern Editor.\n[00-64; min = 00, max = 64].\n'01' also works with 'Note Tracker' (NTr)."
  }
}

WMP_CTRL_LGNX4  = vb:button   { id = 'WMP_CTRL_LGNX4',  width = 30, height = 20, text = 'x4',  notifier = function() vb.views['WMP_CTRL_LGN'].value = 4 vb.views['WMP_CTRL_JMP'].value = 4 end, tooltip = 'Multiplier x4 for Step Length & Jump.' }

WMP_CTRL_LGNX8  = vb:button   { id = 'WMP_CTRL_LGNX8',  width = 30, height = 20, text = 'x8',  notifier = function() vb.views['WMP_CTRL_LGN'].value = 8 vb.views['WMP_CTRL_JMP'].value = 8 end, tooltip = 'Multiplier x8 for Step Length & Jump.' }

WMP_CTRL_LGNX16 = vb:button   { id = 'WMP_CTRL_LGNX16', width = 30, height = 20, text = 'x16', notifier = function() vb.views['WMP_CTRL_LGN'].value =16 vb.views['WMP_CTRL_JMP'].value =16 end, tooltip = 'Multiplier x16 for Step Length & Jump.' }

WMP_CTRL_NOT = vb:row { vb:text { text = 'AutoNote' }, vb:valuebox { id = 'WMP_CTRL_NOT', height = 22, min = 0, max = 119, tooltip = 'Show number of note pressed.\n[0-119]' } } --"bridge valuebox" for note_value (hidden)

--- --- ---

WMP_CTRL_NCM = vb:column {--Note Column Matrix
  vb:row {
    vb:button   { id = 'WMP_CTRL_NC1',  width = 22, height = 12, color = { 0xD6,0x7F,0xFF }, notifier = function() wmp_fn_bt_nc1() end,  tooltip = 'Note Column 1.',  bitmap = 'icons/01-cl-ico.bmp' },
    vb:button   { id = 'WMP_CTRL_NC2',  width = 22, height = 12, color = { 0xD6,0x7F,0xFF }, notifier = function() wmp_fn_bt_nc2() end,  tooltip = 'Note Column 2.',  bitmap = 'icons/02-cl-ico.bmp' },
    vb:button   { id = 'WMP_CTRL_NC3',  width = 22, height = 12, color = { 0xD6,0x7F,0xFF }, notifier = function() wmp_fn_bt_nc3() end,  tooltip = 'Note Column 3.',  bitmap = 'icons/03-cl-ico.bmp' },
    vb:button   { id = 'WMP_CTRL_NC4',  width = 22, height = 12, color = { 0xD6,0x7F,0xFF }, notifier = function() wmp_fn_bt_nc4() end,  tooltip = 'Note Column 4.',  bitmap = 'icons/04-cl-ico.bmp' },
    vb:button   { id = 'WMP_CTRL_NC5',  width = 22, height = 12, color = { 0xD6,0x7F,0xFF }, notifier = function() wmp_fn_bt_nc5() end,  tooltip = 'Note Column 5.',  bitmap = 'icons/05-cl-ico.bmp' },
    vb:button   { id = 'WMP_CTRL_NC6',  width = 22, height = 12, color = { 0xD6,0x7F,0xFF }, notifier = function() wmp_fn_bt_nc6() end,  tooltip = 'Note Column 6.',  bitmap = 'icons/06-cl-ico.bmp' },
  },
  vb:row {
    vb:button   { id = 'WMP_CTRL_NC7',  width = 22, height = 12, color = { 0xD6,0x7F,0xFF }, notifier = function() wmp_fn_bt_nc7() end,  tooltip = 'Note Column 7.',  bitmap = 'icons/07-cl-ico.bmp' },
    vb:button   { id = 'WMP_CTRL_NC8',  width = 22, height = 12, color = { 0xD6,0x7F,0xFF }, notifier = function() wmp_fn_bt_nc8() end,  tooltip = 'Note Column 8.',  bitmap = 'icons/08-cl-ico.bmp' },
    vb:button   { id = 'WMP_CTRL_NC9',  width = 22, height = 12, color = { 0xD6,0x7F,0xFF }, notifier = function() wmp_fn_bt_nc9() end,  tooltip = 'Note Column 9.',  bitmap = 'icons/09-cl-ico.bmp' },
    vb:button   { id = 'WMP_CTRL_NC10', width = 22, height = 12, color = { 0xD6,0x7F,0xFF }, notifier = function() wmp_fn_bt_nc10() end, tooltip = 'Note Column 10.', bitmap = 'icons/10-cl-ico.bmp' },
    vb:button   { id = 'WMP_CTRL_NC11', width = 22, height = 12, color = { 0xD6,0x7F,0xFF }, notifier = function() wmp_fn_bt_nc11() end, tooltip = 'Note Column 11.', bitmap = 'icons/11-cl-ico.bmp' },
    vb:button   { id = 'WMP_CTRL_NC12', width = 22, height = 12, color = { 0xD6,0x7F,0xFF }, notifier = function() wmp_fn_bt_nc12() end, tooltip = 'Note Column 12.', bitmap = 'icons/12-cl-ico.bmp' },
  }
}

WMP_CTRL_DPLY = vb:row { vb:bitmap { bitmap = 'icons/deploy-ico.bmp', mode = 'body_color', height = 24, width = 20 }, vb:valuebox { id ='WMP_CTRL_DPLY', width = 50, height = 22, min = 1, max = 2, value = 1, tooltip = 'Deploy Virtual Piano.\n 01 = plicated\n 02 = deployed (all octaves)',
    notifier = function()
      if     vb.views['WMP_CTRL_DPLY'].value == 1 then
             vb.views['WMP_COLUMN'].height = 1
             vb.views['WMP_VERTICAL'].height = 1
      elseif vb.views['WMP_CTRL_DPLY'].value == 2 then
             vb.views['WMP_COLUMN'].height = 1
             vb.views['WMP_VERTICAL'].height = 140       
      end
    end
  }
}


--[[

unicode:  ▲ ▼ ◀ ▶  ◄◄  ►►  ●  ▶ ┇▶  ■  ♞

]]


--------------------------------------------------------------------------------
-- REACT PIANO LIGHTING
--------------------------------------------------------------------------------

function key_lighting( song, spi, sti, sli, snci )
  song = renoise.song()
  spi, sti, sli, snci = song.selected_pattern_index, song.selected_track_index, song.selected_line_index, song.selected_note_column_index
  if song.transport.playing == true then
    if song.patterns[ spi ].tracks[ sti ].lines[ sli ].note_columns[ snci ].note_value == 0 then
      vb.views['WMP_KEY_00'].color = { 0x90,0x00,0x40 }
    end
  end
end

function pno_light_bl_t()
  vb.views['WMP_PNO_01'].color = { 0x00,0x00,0xA0 }
end

function pno_light_bl_f()
  vb.views['WMP_PNO_01'].color = { 0x00,0x00,0x40 }
end

function pno_light_rd_t()
  vb.views['WMP_PNO_02'].color = { 0xA5,0x00,0x00 }
end

function pno_light_rd_f()
  vb.views['WMP_PNO_02'].color = { 0x45,0x00,0x00 }
end

function pno_light_gr_t()
  vb.views['WMP_PNO_03'].color = { 0x00,0x90,0x00 }
end

function pno_light_gr_f()
  vb.views['WMP_PNO_03'].color = { 0x00,0x30,0x00 }
end

function pno_light_gy_t()
  vb.views['WMP_PNO_04'].color = { 0xDD,0xDD,0xDD }
end

function pno_light_gy_f()
  vb.views['WMP_PNO_04'].color = { 0x45,0x45,0x45 }
end



--------------------------------------------------------------------------------
--BUTTON KEYS (NOTES)
--------------------------------------------------------------------------------
WMP_PNO_01 = vb:button {
  id = 'WMP_PNO_01',
  active = false,
  width = 676,
  height = 12,
  color = { 0x00,0x00,0x40 },
}
WMP_PNO_02 = vb:button {
  id = 'WMP_PNO_02',
  active = false,
  width = 676,
  height = 12,
  color = { 0x45,0x00,0x00 },
}
WMP_PNO_03 = vb:button {
  id = 'WMP_PNO_03',
  active = false,
  width = 337,
  height = 12,
  color = { 0x00,0x30,0x00 },
}

WMP_PNO_04 = vb:button {
  id = 'WMP_PNO_04',
  active = false,
  width = 44,
  height = 12,
  color = { 0x45,0x45,0x45 },
}

WMP_SEP_01 = vb:horizontal_aligner { width = 1 }
WMP_SEP_02 = vb:horizontal_aligner { width = 1 }
WMP_SEP_03 = vb:horizontal_aligner { width = 1 }
WMP_SEP_04 = vb:horizontal_aligner { width = 1 }
WMP_SEP_05 = vb:horizontal_aligner { width = 1 }
WMP_SEP_06 = vb:horizontal_aligner { width = 1 }
WMP_SEP_07 = vb:horizontal_aligner { width = 1 }
WMP_SEP_08 = vb:horizontal_aligner { width = 1 }
WMP_SEP_09 = vb:horizontal_aligner { width = 1 }
WMP_SEP_10 = vb:horizontal_aligner { width = 4 }
WMP_SEP_11 = vb:horizontal_aligner { width = 4 }
WMP_SEP_12 = vb:horizontal_aligner { width = 4 }
WMP_SEP_13 = vb:horizontal_aligner { width = 4 }

--- --- octave 0
WMP_KEY_00 = vb:button {
  id = 'WMP_KEY_00',
  tooltip = 'C-0',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 0  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/c-0.png',
}
WMP_KEY_01 = vb:button {
  id = 'WMP_KEY_01',
  tooltip = 'C#0',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 1  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_02 = vb:button {
  id = 'WMP_KEY_02',
  tooltip = 'D-0',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 2  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/d-.png',
}
WMP_KEY_03 = vb:button {
  id = 'WMP_KEY_03',
  tooltip = 'D#0',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 3  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_04 = vb:button {
  id = 'WMP_KEY_04',
  tooltip = 'E-0',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 4  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/e-.png',
}
WMP_KEY_05 = vb:button {
  id = 'WMP_KEY_05',
  tooltip = 'F-0',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 5  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/f-.png',
}
WMP_KEY_06 = vb:button {
  id = 'WMP_KEY_06',
  tooltip = 'F#0',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 6  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_07 = vb:button {
  id = 'WMP_KEY_07',
  tooltip = 'G-0',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 7  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/g-.png',
}
WMP_KEY_08 = vb:button {
  id = 'WMP_KEY_08',
  tooltip = 'G#0',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 8  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_09 = vb:button {
  id = 'WMP_KEY_09',
  tooltip = 'A-0',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 9  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/a-.png',
}
WMP_KEY_10 = vb:button {
  id = 'WMP_KEY_10',
  tooltip = 'A#0',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 10  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_11 = vb:button {
  id = 'WMP_KEY_11',
  tooltip = 'B-0',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 11  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/b-.png',
}

--- --- octave 1
WMP_KEY_12 = vb:button {
  id = 'WMP_KEY_12',
  tooltip = 'C-1',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  bitmap = 'icons/c-1.png',
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 12  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/c-1.png',
}
WMP_KEY_13 = vb:button {
  id = 'WMP_KEY_13',
  tooltip = 'C#1',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 13  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_14 = vb:button {
  id = 'WMP_KEY_14',
  tooltip = 'D-1',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  bitmap = 'icons/d-.png',
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 14  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_15 = vb:button {
  id = 'WMP_KEY_15',
  tooltip = 'D#1',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 15  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_16 = vb:button {
  id = 'WMP_KEY_16',
  tooltip = 'E-1',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 16  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/e-.png',
}
WMP_KEY_17 = vb:button {
  id = 'WMP_KEY_17',
  tooltip = 'F-1',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 17  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/f-.png',
}
WMP_KEY_18 = vb:button {
  id = 'WMP_KEY_18',
  tooltip = 'F#1',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 18  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_19 = vb:button {
  id = 'WMP_KEY_19',
  tooltip = 'G-1',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 19  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/g-.png',
}
WMP_KEY_20 = vb:button {
  id = 'WMP_KEY_20',
  tooltip = 'G#1',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 20  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_21 = vb:button {
  id = 'WMP_KEY_21',
  tooltip = 'A-1',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 21  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/a-.png',
}
WMP_KEY_22 = vb:button {
  id = 'WMP_KEY_22',
  tooltip = 'A#1',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 22  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_23 = vb:button {
  id = 'WMP_KEY_23',
  tooltip = 'B-1',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 23  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/b-.png',
}

--- --- octave 2
WMP_KEY_24 = vb:button {
  id = 'WMP_KEY_24',
  tooltip = 'C-2',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 24  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/c-2.png',
}
WMP_KEY_25 = vb:button {
  id = 'WMP_KEY_25',
  tooltip = 'C#2',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 25  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_26 = vb:button {
  id = 'WMP_KEY_26',
  tooltip = 'D-2',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 26  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/d-.png',
}
WMP_KEY_27 = vb:button {
  id = 'WMP_KEY_27',
  tooltip = 'D#2',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 27  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_28 = vb:button {
  id = 'WMP_KEY_28',
  tooltip = 'E-2',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 28  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/e-.png',
}
WMP_KEY_29 = vb:button {
  id = 'WMP_KEY_29',
  tooltip = 'F-2',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 29  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/f-.png',
}
WMP_KEY_30 = vb:button {
  id = 'WMP_KEY_30',
  tooltip = 'F#2',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 30  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_31 = vb:button {
  id = 'WMP_KEY_31',
  tooltip = 'G-2',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 31  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/g-.png',
}
WMP_KEY_32 = vb:button {
  id = 'WMP_KEY_32',
  tooltip = 'G#2',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 32  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_33 = vb:button {
  id = 'WMP_KEY_33',
  tooltip = 'A-2',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 33  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/a-.png',
}
WMP_KEY_34 = vb:button {
  id = 'WMP_KEY_34',
  tooltip = 'A#2',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 34  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_35 = vb:button {
  id = 'WMP_KEY_35',
  tooltip = 'B-2',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 35  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/b-.png',
}

--- --- octave 3
WMP_KEY_36 = vb:button {
  id = 'WMP_KEY_36',
  tooltip = 'C-3',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 36  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/c-3.png',
}
WMP_KEY_37 = vb:button {
  id = 'WMP_KEY_37',
  tooltip = 'C#3',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 37  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_38 = vb:button {
  id = 'WMP_KEY_38',
  tooltip = 'D-3',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 38  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/d-.png',
}
WMP_KEY_39 = vb:button {
  id = 'WMP_KEY_39',
  tooltip = 'D#3',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 39  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_40 = vb:button {
  id = 'WMP_KEY_40',
  tooltip = 'E-3',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 40  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/e-.png',
}
WMP_KEY_41 = vb:button {
  id = 'WMP_KEY_41',
  tooltip = 'F-3',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 41  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/f-.png',
}
WMP_KEY_42 = vb:button {
  id = 'WMP_KEY_42',
  tooltip = 'F#3',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 42  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_43 = vb:button {
  id = 'WMP_KEY_43',
  tooltip = 'G-3',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 43  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/g-.png',
}
WMP_KEY_44 = vb:button {
  id = 'WMP_KEY_44',
  tooltip = 'G#3',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 44  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_45 = vb:button {
  id = 'WMP_KEY_45',
  tooltip = 'A-3',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 45  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/a-.png',
}
WMP_KEY_46 = vb:button {
  id = 'WMP_KEY_46',
  tooltip = 'A#3',
  width = 14,
  height = 36,
  color = { 0x26,0x26,0x76 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 46  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
}
WMP_KEY_47 = vb:button {
  id = 'WMP_KEY_47',
  tooltip = 'B-3',
  width = 14,
  height = 48,
  color = { 0xCF,0xCF,0xFF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 47  repeat_key() wmp_fn_bt_pres() pno_light_bl_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_bl_f() end,
  bitmap = 'icons/b-.png',
}

--- --- octave 4
WMP_KEY_48 = vb:button {
  id = 'WMP_KEY_48',
  tooltip = 'C-4',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 48  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/c-4.png',
}
WMP_KEY_49 = vb:button {
  id = 'WMP_KEY_49',
  tooltip = 'C#4',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 49  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_50 = vb:button {
  id = 'WMP_KEY_50',
  tooltip = 'D-4',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 50  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/d-.png',
}
WMP_KEY_51 = vb:button {
  id = 'WMP_KEY_51',
  tooltip = 'D#4',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 51  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_52 = vb:button {
  id = 'WMP_KEY_52',
  tooltip = 'E-4',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 52  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/e-.png',
}
WMP_KEY_53 = vb:button {
  id = 'WMP_KEY_53',
  tooltip = 'F-4',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 53  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/f-.png',
}
WMP_KEY_54 = vb:button {
  id = 'WMP_KEY_54',
  tooltip = 'F#4',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 54  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_55 = vb:button {
  id = 'WMP_KEY_55',
  tooltip = 'G-4',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 55  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/g-.png',
}
WMP_KEY_56 = vb:button {
  id = 'WMP_KEY_56',
  tooltip = 'G#4',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 56  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_57 = vb:button {
  id = 'WMP_KEY_57',
  tooltip = 'A-4',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 57  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/a-.png',
}
WMP_KEY_58 = vb:button {
  id = 'WMP_KEY_58',
  tooltip = 'A#4',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 58  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_59 = vb:button {
  id = 'WMP_KEY_59',
  tooltip = 'B-4',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 59  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/b-.png',
}

--- --- octave 5
WMP_KEY_60 = vb:button {
  id = 'WMP_KEY_60',
  tooltip = 'C-5',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 60  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/c-5.png',
}
WMP_KEY_61 = vb:button {
  id = 'WMP_KEY_61',
  tooltip = 'C#5',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 61  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_62 = vb:button {
  id = 'WMP_KEY_62',
  tooltip = 'D-5',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 62  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/d-.png',
}
WMP_KEY_63 = vb:button {
  id = 'WMP_KEY_63',
  tooltip = 'D#5',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 63  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_64 = vb:button {
  id = 'WMP_KEY_64',
  tooltip = 'E-5',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 64  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/e-.png',
}
WMP_KEY_65 = vb:button {
  id = 'WMP_KEY_65',
  tooltip = 'F-5',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 65  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/f-.png',
}
WMP_KEY_66 = vb:button {
  id = 'WMP_KEY_66',
  tooltip = 'F#5',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 66  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_67 = vb:button {
  id = 'WMP_KEY_67',
  tooltip = 'G-5',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 67  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/g-.png',
}
WMP_KEY_68 = vb:button {
  id = 'WMP_KEY_68',
  tooltip = 'G#5',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 68  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_69 = vb:button {
  id = 'WMP_KEY_69',
  tooltip = 'A-5',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 69  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/a-.png',
}
WMP_KEY_70 = vb:button {
  id = 'WMP_KEY_70',
  tooltip = 'A#5',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 70  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_71 = vb:button {
  id = 'WMP_KEY_71',
  tooltip = 'B-5',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 71  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/b-.png',
}

--- --- octave 6
WMP_KEY_72 = vb:button {
  id = 'WMP_KEY_72',
  tooltip = 'C-6',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 72  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/c-6.png',
}
WMP_KEY_73 = vb:button {
  id = 'WMP_KEY_73',
  tooltip = 'C#6',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 73  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_74 = vb:button {
  id = 'WMP_KEY_74',
  tooltip = 'D-6',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 74  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/d-.png',
}
WMP_KEY_75 = vb:button {
  id = 'WMP_KEY_75',
  tooltip = 'D#6',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 75  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_76 = vb:button {
  id = 'WMP_KEY_76',
  tooltip = 'E-6',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 76  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/e-.png',
}
WMP_KEY_77 = vb:button {
  id = 'WMP_KEY_77',
  tooltip = 'F-6',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 77  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/f-.png',
}
WMP_KEY_78 = vb:button {
  id = 'WMP_KEY_78',
  tooltip = 'F#6',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 78  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_79 = vb:button {
  id = 'WMP_KEY_79',
  tooltip = 'G-6',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 79  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/g-.png',
}
WMP_KEY_80 = vb:button {
  id = 'WMP_KEY_80',
  tooltip = 'G#6',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 80  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_81 = vb:button {
  id = 'WMP_KEY_81',
  tooltip = 'A-6',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 81  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/a-.png',
}
WMP_KEY_82 = vb:button {
  id = 'WMP_KEY_82',
  tooltip = 'A#6',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 82  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_83 = vb:button {
  id = 'WMP_KEY_83',
  tooltip = 'B-6',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 83  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/b-.png',
}

--- --- octave 7
WMP_KEY_84 = vb:button {
  id = 'WMP_KEY_84',
  tooltip = 'C-7',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 84  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/c-7.png',
}
WMP_KEY_85 = vb:button {
  id = 'WMP_KEY_85',
  tooltip = 'C#7',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 85  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_86 = vb:button {
  id = 'WMP_KEY_86',
  tooltip = 'D-7',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 86  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/d-.png',
}
WMP_KEY_87 = vb:button {
  id = 'WMP_KEY_87',
  tooltip = 'D#7',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 87  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_88 = vb:button {
  id = 'WMP_KEY_88',
  tooltip = 'E-7',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 88  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/e-.png',
}
WMP_KEY_89 = vb:button {
  id = 'WMP_KEY_89',
  tooltip = 'F-7',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 89  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/f-.png',
}
WMP_KEY_90 = vb:button {
  id = 'WMP_KEY_90',
  tooltip = 'F#7',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 90  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_91 = vb:button {
  id = 'WMP_KEY_91',
  tooltip = 'G-7',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 91  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/g-.png',
}
WMP_KEY_92 = vb:button {
  id = 'WMP_KEY_92',
  tooltip = 'G#7',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 92  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_93 = vb:button {
  id = 'WMP_KEY_93',
  tooltip = 'A-7',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 93  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/a-.png',
}
WMP_KEY_94 = vb:button {
  id = 'WMP_KEY_94',
  tooltip = 'A#7',
  width = 14,
  height = 36,
  color = { 0x56,0x26,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 94  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
}
WMP_KEY_95 = vb:button {
  id = 'WMP_KEY_95',
  tooltip = 'B-7',
  width = 14,
  height = 48,
  color = { 0xFF,0xCF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 95  repeat_key() wmp_fn_bt_pres() pno_light_rd_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_rd_f() end,
  bitmap = 'icons/b-.png',
}

--- --- octave 8
WMP_KEY_96 = vb:button {
  id = 'WMP_KEY_96',
  tooltip = 'C-8',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 96  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/c-8.png',
}
WMP_KEY_97 = vb:button {
  id = 'WMP_KEY_97',
  tooltip = 'C#8',
  width = 14,
  height = 36,
  color = { 0x26,0x46,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 97  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
}
WMP_KEY_98 = vb:button {
  id = 'WMP_KEY_98',
  tooltip = 'D-8',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 98  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/d-.png',
}
WMP_KEY_99 = vb:button {
  id = 'WMP_KEY_99',
  tooltip = 'D#8',
  width = 14,
  height = 36,
  color = { 0x26,0x46,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 99  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
}
WMP_KEY_100 = vb:button {
  id = 'WMP_KEY_100',
  tooltip = 'E-8',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 100  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/e-.png',
}
WMP_KEY_101 = vb:button {
  id = 'WMP_KEY_101',
  tooltip = 'F-8',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 101  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/f-.png',
}
WMP_KEY_102 = vb:button {
  id = 'WMP_KEY_102',
  tooltip = 'F#8',
  width = 14,
  height = 36,
  color = { 0x26,0x46,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 102  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
}
WMP_KEY_103 = vb:button {
  id = 'WMP_KEY_103',
  tooltip = 'G-8',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 103  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/g-.png',
}
WMP_KEY_104 = vb:button {
  id = 'WMP_KEY_104',
  tooltip = 'G#8',
  width = 14,
  height = 36,
  color = { 0x26,0x46,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 104  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
}
WMP_KEY_105 = vb:button {
  id = 'WMP_KEY_105',
  tooltip = 'A-8',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 105  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/a-.png',
}
WMP_KEY_106 = vb:button {
  id = 'WMP_KEY_106',
  tooltip = 'A#8',
  width = 14,
  height = 36,
  color = { 0x26,0x46,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 106  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
}
WMP_KEY_107 = vb:button {
  id = 'WMP_KEY_107',
  tooltip = 'B-8',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 107  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/b-.png',
}

--- --- octave 9
WMP_KEY_108 = vb:button {
  id = 'WMP_KEY_108',
  tooltip = 'C-9',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 108  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/c-9.png',
}
WMP_KEY_109 = vb:button {
  id = 'WMP_KEY_109',
  tooltip = 'C#9',
  width = 14,
  height = 36,
  color = { 0x26,0x46,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 109  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
}
WMP_KEY_110 = vb:button {
  id = 'WMP_KEY_110',
  tooltip = 'D-9',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 110  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/d-.png',
}
WMP_KEY_111 = vb:button {
  id = 'WMP_KEY_111',
  tooltip = 'D#9',
  width = 14,
  height = 36,
  color = { 0x26,0x46,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 111  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
}
WMP_KEY_112 = vb:button {
  id = 'WMP_KEY_112',
  tooltip = 'E-9',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 112  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/e-.png',
}
WMP_KEY_113 = vb:button {
  id = 'WMP_KEY_113',
  tooltip = 'F-9',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 113  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/f-.png',
}
WMP_KEY_114 = vb:button {
  id = 'WMP_KEY_114',
  tooltip = 'F#9',
  width = 14,
  height = 36,
  color = { 0x26,0x46,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 114  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
}
WMP_KEY_115 = vb:button {
  id = 'WMP_KEY_115',
  tooltip = 'G-9',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 115  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/g-.png',
}
WMP_KEY_116 = vb:button {
  id = 'WMP_KEY_116',
  tooltip = 'G#9',
  width = 14,
  height = 36,
  color = { 0x26,0x46,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 116  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
}
WMP_KEY_117 = vb:button {
  id = 'WMP_KEY_117',
  tooltip = 'A-9',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 117  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/a-.png',
}
WMP_KEY_118 = vb:button {
  id = 'WMP_KEY_118',
  tooltip = 'A#9',
  width = 14,
  height = 36,
  color = { 0x26,0x46,0x26 },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 118  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
}
WMP_KEY_119 = vb:button {
  id = 'WMP_KEY_119',
  tooltip = 'B-9',
  width = 14,
  height = 48,
  color = { 0xCF,0xFF,0xCF },
  pressed =  function() vb.views['WMP_CTRL_NOT'].value = 119  repeat_key() wmp_fn_bt_pres() pno_light_gr_t() end,
  released = function() repeat_key(true) wmp_fn_bt_rel() pno_light_gr_f() end,
  bitmap = 'icons/b-.png',
}
-- note-off
WMP_KEY_120 = vb:button {
  id = 'WMP_KEY_120',
  tooltip = 'Note-OFF',
  width = 14,
  height = 48,
  color = { 0xCC,0xCC,0xCC },
  pressed =  function() repeat_key_120() pno_light_gy_t() end,
  released = function() repeat_key_120(true) pno_light_gy_f() end,
  bitmap = 'icons/off.png',
}
-- empty
WMP_KEY_121 = vb:button {
  id = 'WMP_KEY_121',
  tooltip = 'Note-EMPTY',
  width = 14,
  height = 48,
  color = { 0xCC,0xCC,0xCC },
  pressed =  function() repeat_key_121() pno_light_gy_t() end,
  released = function() repeat_key_121(true) pno_light_gy_f() end,
  bitmap = 'icons/emp.png',
}
-- instrument
WMP_KEY_122 = vb:button {
  id = 'WMP_KEY_122',
  tooltip = 'Instrument Only\n(Work also with NTr).',
  width = 14,
  height = 48,
  color = { 0xCC,0xCC,0xCC },
  pressed =  function() repeat_key_122() pno_light_gy_t() end,
  released = function() repeat_key_122(true) pno_light_gy_f() end,
  bitmap = 'icons/ins.png',
}


--------------------------------------------------------------------------------
-- GUI - TOOL 08
--------------------------------------------------------------------------------
WMP =
--vb:column {
vb:vertical_aligner {
  id = 'WMP_COLUMN',
  width = 678,
  vb:row {
    --height = 20,
    vb:horizontal_aligner {
      width = 304,
      mode = 'center',
      vb:bitmap {
        bitmap = 'icons/mouse-ico.bmp',
        mode = 'body_color',
        height = 22,
      },
      vb:text { font = 'bold', text = 'WMP: Write Mouse Piano', height = 22  },
      vb:text { text = '   v1.0b', height = 22  },
    },
    vb:row { WMP_CTRL_INS, WMP_CTRL_NME, WMP_SEP_10, WMP_CTRL_DPLY, WMP_CTRL_DEF }
  },
  vb:horizontal_aligner { height = 2 },

---------------
vb:vertical_aligner { --height = 1, --height deploy
  id = 'WMP_VERTICAL',
  height = 1,
  vb:horizontal_aligner { WMP_PNO_01 },
  vb:horizontal_aligner {
    WMP_KEY_00, WMP_KEY_01, WMP_KEY_02, WMP_KEY_03, WMP_KEY_04, WMP_KEY_05, WMP_KEY_06, WMP_KEY_07, WMP_KEY_08, WMP_KEY_09, WMP_KEY_10, WMP_KEY_11, WMP_SEP_01, --octave 0
    WMP_KEY_12, WMP_KEY_13, WMP_KEY_14, WMP_KEY_15, WMP_KEY_16, WMP_KEY_17, WMP_KEY_18, WMP_KEY_19, WMP_KEY_20, WMP_KEY_21, WMP_KEY_22, WMP_KEY_23, WMP_SEP_02, --octave 1
    WMP_KEY_24, WMP_KEY_25, WMP_KEY_26, WMP_KEY_27, WMP_KEY_28, WMP_KEY_29, WMP_KEY_30, WMP_KEY_31, WMP_KEY_32, WMP_KEY_33, WMP_KEY_34, WMP_KEY_35, WMP_SEP_03, --octave 2
    WMP_KEY_36, WMP_KEY_37, WMP_KEY_38, WMP_KEY_39, WMP_KEY_40, WMP_KEY_41, WMP_KEY_42, WMP_KEY_43, WMP_KEY_44, WMP_KEY_45, WMP_KEY_46, WMP_KEY_47,             --octave 3
  },
  vb:horizontal_aligner { height = 10 },
  vb:horizontal_aligner { WMP_PNO_02 },
  vb:horizontal_aligner { 
    WMP_KEY_48, WMP_KEY_49, WMP_KEY_50, WMP_KEY_51, WMP_KEY_52, WMP_KEY_53, WMP_KEY_54, WMP_KEY_55, WMP_KEY_56, WMP_KEY_57, WMP_KEY_58, WMP_KEY_59, WMP_SEP_04, --octave 4
    WMP_KEY_60, WMP_KEY_61, WMP_KEY_62, WMP_KEY_63, WMP_KEY_64, WMP_KEY_65, WMP_KEY_66, WMP_KEY_67, WMP_KEY_68, WMP_KEY_69, WMP_KEY_70, WMP_KEY_71, WMP_SEP_05, --octave 5
    WMP_KEY_72, WMP_KEY_73, WMP_KEY_74, WMP_KEY_75, WMP_KEY_76, WMP_KEY_77, WMP_KEY_78, WMP_KEY_79, WMP_KEY_80, WMP_KEY_81, WMP_KEY_82, WMP_KEY_83, WMP_SEP_06, --octave 6
    WMP_KEY_84, WMP_KEY_85, WMP_KEY_86, WMP_KEY_87, WMP_KEY_88, WMP_KEY_89, WMP_KEY_90, WMP_KEY_91, WMP_KEY_92, WMP_KEY_93, WMP_KEY_94, WMP_KEY_95,             --octave 7
  },
  vb:horizontal_aligner { height = 10 }
},
---------------
  vb:horizontal_aligner {
    vb:column {
      vb:horizontal_aligner { WMP_PNO_03, WMP_PNO_04 },
      vb:horizontal_aligner {
      
        WMP_KEY_96,  WMP_KEY_97,  WMP_KEY_98,  WMP_KEY_99,  WMP_KEY_100, WMP_KEY_101, WMP_KEY_102, WMP_KEY_103, WMP_KEY_104, WMP_KEY_105, WMP_KEY_106, WMP_KEY_107, WMP_SEP_07,  --octave 8
        WMP_KEY_108, WMP_KEY_109, WMP_KEY_110, WMP_KEY_111, WMP_KEY_112, WMP_KEY_113, WMP_KEY_114, WMP_KEY_115, WMP_KEY_116, WMP_KEY_117, WMP_KEY_118, WMP_KEY_119, WMP_SEP_08,  --octave 9
        WMP_KEY_120, WMP_KEY_121, WMP_KEY_122 --"note-off" , "empty"
      },
      vb:horizontal_aligner { height = 7 },
      vb:horizontal_aligner {                                                  
        WMP_CTRL_FCS, WMP_CTRL_PLZ, WMP_CTRL_PLY, WMP_CTRL_STP, WMP_SEP_11, WMP_CTRL_VEL, WMP_SEP_12, WMP_CTRL_CLR_NC, WMP_CTRL_CLR_FX, WMP_SEP_13, WMP_CTRL_BT_NCTR, WMP_CTRL_BT_VAL, WMP_CTRL_BT_NTR
      }
    },
    WMP_SEP_09,
    vb:column {
      style = 'panel',--panel border body plain group panel
      margin = 4,
      width = 300,
      --- ---
      vb:horizontal_aligner {
        vb:column {
          vb:horizontal_aligner {
            WMP_CTRL_LGN, WMP_CTRL_LGNX4, WMP_CTRL_LGNX8, WMP_CTRL_LGNX16
          },
          vb:horizontal_aligner {
            WMP_CTRL_PP_VOL, WMP_CTRL_PP_PAN, WMP_CTRL_PP_DLY
          },
          vb:horizontal_aligner {
            WMP_CTRL_BT_VOL, WMP_CTRL_BT_PAN, WMP_CTRL_BT_DLY
          },
          vb:column {
            vb:horizontal_aligner {
              WMP_CTRL_PP_EFF, WMP_CTRL_PP_AMO, WMP_CTRL_BT_EFF
            }
          }
        },
        --vb:column{ width = 2 },
        vb:column {
          vb:row {
            vb:column {  
              WMP_CTRL_TRL,
              vb:row {
                WMP_CTRL_LF2, WMP_CTRL_LFT
              }
            },
            vb:column {
              WMP_CTRL_JMP,
              vb:row { WMP_CTRL_UP,  WMP_CTRL_UP2 },
              vb:row { WMP_CTRL_DWN, WMP_CTRL_DW2 },
            },
            vb:column {
              WMP_CTRL_TRR,
              vb:row {
                WMP_CTRL_RGT, WMP_CTRL_RG2
              }
            }
          },
          WMP_CTRL_NCM
        }
      }
    }
  }
}
